import typing, abc
from System.Collections import ICollection, IEnumerator, ReadOnlyCollectionBase, IDictionary, IList, IComparer
from System import Array, Array_1, Attribute, EventArgs, MarshalByRefObject, IDisposable, IServiceProvider, IAsyncResult, Delegate, EventHandler, Exception
from System.Globalization import CultureInfo
from System.Runtime.InteropServices import ExternalException
from System.Reflection import MethodBase
from System.Runtime.Serialization import SerializationInfo, StreamingContext

class AttributeCollection(ICollection):
    def __init__(self, attributes: Array_1[Attribute]) -> None: ...
    Empty : AttributeCollection
    @property
    def Count(self) -> int: ...
    @property
    def Item(self) -> Attribute: ...
    @property
    def Item(self) -> Attribute: ...
    def CopyTo(self, array: Array, index: int) -> None: ...
    @staticmethod
    def FromExisting(existing: AttributeCollection, newAttributes: Array_1[Attribute]) -> AttributeCollection: ...
    def GetEnumerator(self) -> IEnumerator: ...
    # Skipped Contains due to it being static, abstract and generic.

    Contains : Contains_MethodGroup
    class Contains_MethodGroup:
        @typing.overload
        def __call__(self, attributes: Array_1[Attribute]) -> bool:...
        @typing.overload
        def __call__(self, attribute: Attribute) -> bool:...

    # Skipped Matches due to it being static, abstract and generic.

    Matches : Matches_MethodGroup
    class Matches_MethodGroup:
        @typing.overload
        def __call__(self, attributes: Array_1[Attribute]) -> bool:...
        @typing.overload
        def __call__(self, attribute: Attribute) -> bool:...



class BackgroundWorker(Component):
    def __init__(self) -> None: ...
    @property
    def CancellationPending(self) -> bool: ...
    @property
    def Container(self) -> IContainer: ...
    @property
    def IsBusy(self) -> bool: ...
    @property
    def Site(self) -> ISite: ...
    @Site.setter
    def Site(self, value: ISite) -> ISite: ...
    @property
    def WorkerReportsProgress(self) -> bool: ...
    @WorkerReportsProgress.setter
    def WorkerReportsProgress(self, value: bool) -> bool: ...
    @property
    def WorkerSupportsCancellation(self) -> bool: ...
    @WorkerSupportsCancellation.setter
    def WorkerSupportsCancellation(self, value: bool) -> bool: ...
    def CancelAsync(self) -> None: ...
    # Skipped ReportProgress due to it being static, abstract and generic.

    ReportProgress : ReportProgress_MethodGroup
    class ReportProgress_MethodGroup:
        @typing.overload
        def __call__(self, percentProgress: int) -> None:...
        @typing.overload
        def __call__(self, percentProgress: int, userState: typing.Any) -> None:...

    # Skipped RunWorkerAsync due to it being static, abstract and generic.

    RunWorkerAsync : RunWorkerAsync_MethodGroup
    class RunWorkerAsync_MethodGroup:
        @typing.overload
        def __call__(self) -> None:...
        @typing.overload
        def __call__(self, argument: typing.Any) -> None:...



class CancelEventArgs(EventArgs):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, cancel: bool) -> None: ...
    @property
    def Cancel(self) -> bool: ...
    @Cancel.setter
    def Cancel(self, value: bool) -> bool: ...


class Component(MarshalByRefObject, IComponent):
    def __init__(self) -> None: ...
    @property
    def Container(self) -> IContainer: ...
    @property
    def Site(self) -> ISite: ...
    @Site.setter
    def Site(self, value: ISite) -> ISite: ...
    def Dispose(self) -> None: ...
    def ToString(self) -> str: ...


class ComponentCollection(ReadOnlyCollectionBase):
    def __init__(self, components: Array_1[IComponent]) -> None: ...
    @property
    def Count(self) -> int: ...
    @property
    def Item(self) -> IComponent: ...
    @property
    def Item(self) -> IComponent: ...
    def CopyTo(self, array: Array_1[IComponent], index: int) -> None: ...


class DefaultValueAttribute(Attribute):
    # Constructor .ctor(value : Int16) was skipped since it collides with above method
    # Constructor .ctor(value : Int32) was skipped since it collides with above method
    # Constructor .ctor(value : Int64) was skipped since it collides with above method
    # Constructor .ctor(value : Single) was skipped since it collides with above method
    # Constructor .ctor(value : Double) was skipped since it collides with above method
    # Constructor .ctor(value : Boolean) was skipped since it collides with above method
    # Constructor .ctor(value : String) was skipped since it collides with above method
    # Constructor .ctor(value : SByte) was skipped since it collides with above method
    # Constructor .ctor(value : UInt16) was skipped since it collides with above method
    # Constructor .ctor(value : UInt32) was skipped since it collides with above method
    # Constructor .ctor(value : UInt64) was skipped since it collides with above method
    @typing.overload
    def __init__(self, type: typing.Type[typing.Any], value: str) -> None: ...
    @typing.overload
    def __init__(self, value: str) -> None: ...
    @typing.overload
    def __init__(self, value: int) -> None: ...
    @typing.overload
    def __init__(self, value: typing.Any) -> None: ...
    @property
    def TypeId(self) -> typing.Any: ...
    @property
    def Value(self) -> typing.Any: ...
    def Equals(self, obj: typing.Any) -> bool: ...
    def GetHashCode(self) -> int: ...


class DesignerSerializationVisibility(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Hidden : DesignerSerializationVisibility # 0
    Visible : DesignerSerializationVisibility # 1
    Content : DesignerSerializationVisibility # 2


class DoWorkEventArgs(CancelEventArgs):
    def __init__(self, argument: typing.Any) -> None: ...
    @property
    def Argument(self) -> typing.Any: ...
    @property
    def Cancel(self) -> bool: ...
    @Cancel.setter
    def Cancel(self, value: bool) -> bool: ...
    @property
    def Result(self) -> typing.Any: ...
    @Result.setter
    def Result(self, value: typing.Any) -> typing.Any: ...


class EditorBrowsableAttribute(Attribute):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, state: EditorBrowsableState) -> None: ...
    @property
    def State(self) -> EditorBrowsableState: ...
    @property
    def TypeId(self) -> typing.Any: ...
    def Equals(self, obj: typing.Any) -> bool: ...
    def GetHashCode(self) -> int: ...


class EditorBrowsableState(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Always : EditorBrowsableState # 0
    Never : EditorBrowsableState # 1
    Advanced : EditorBrowsableState # 2


class IComponent(IDisposable, typing.Protocol):
    @property
    def Site(self) -> ISite: ...
    @Site.setter
    def Site(self, value: ISite) -> ISite: ...


class IContainer(IDisposable, typing.Protocol):
    @property
    def Components(self) -> ComponentCollection: ...
    @abc.abstractmethod
    def Remove(self, component: IComponent) -> None: ...
    # Skipped Add due to it being static, abstract and generic.

    Add : Add_MethodGroup
    class Add_MethodGroup:
        @typing.overload
        def __call__(self, component: IComponent) -> None:...
        @typing.overload
        def __call__(self, component: IComponent, name: str) -> None:...



class INotifyPropertyChanged(typing.Protocol):
    pass


class ISite(IServiceProvider, typing.Protocol):
    @property
    def Component(self) -> IComponent: ...
    @property
    def Container(self) -> IContainer: ...
    @property
    def DesignMode(self) -> bool: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...


class ISynchronizeInvoke(typing.Protocol):
    @property
    def InvokeRequired(self) -> bool: ...
    @abc.abstractmethod
    def BeginInvoke(self, method: Delegate, args: Array_1[typing.Any]) -> IAsyncResult: ...
    @abc.abstractmethod
    def EndInvoke(self, result: IAsyncResult) -> typing.Any: ...
    @abc.abstractmethod
    def Invoke(self, method: Delegate, args: Array_1[typing.Any]) -> typing.Any: ...


class ITypeDescriptorContext(IServiceProvider, typing.Protocol):
    @property
    def Container(self) -> IContainer: ...
    @property
    def Instance(self) -> typing.Any: ...
    @property
    def PropertyDescriptor(self) -> PropertyDescriptor: ...
    @abc.abstractmethod
    def OnComponentChanged(self) -> None: ...
    @abc.abstractmethod
    def OnComponentChanging(self) -> bool: ...


class MemberDescriptor(abc.ABC):
    @property
    def Attributes(self) -> AttributeCollection: ...
    @property
    def Category(self) -> str: ...
    @property
    def Description(self) -> str: ...
    @property
    def DesignTimeOnly(self) -> bool: ...
    @property
    def DisplayName(self) -> str: ...
    @property
    def IsBrowsable(self) -> bool: ...
    @property
    def Name(self) -> str: ...
    def Equals(self, obj: typing.Any) -> bool: ...
    def GetHashCode(self) -> int: ...


class PropertyDescriptor(MemberDescriptor):
    @property
    def Attributes(self) -> AttributeCollection: ...
    @property
    def Category(self) -> str: ...
    @property
    def ComponentType(self) -> typing.Type[typing.Any]: ...
    @property
    def Converter(self) -> TypeConverter: ...
    @property
    def Description(self) -> str: ...
    @property
    def DesignTimeOnly(self) -> bool: ...
    @property
    def DisplayName(self) -> str: ...
    @property
    def IsBrowsable(self) -> bool: ...
    @property
    def IsLocalizable(self) -> bool: ...
    @property
    def IsReadOnly(self) -> bool: ...
    @property
    def Name(self) -> str: ...
    @property
    def PropertyType(self) -> typing.Type[typing.Any]: ...
    @property
    def SerializationVisibility(self) -> DesignerSerializationVisibility: ...
    @property
    def SupportsChangeEvents(self) -> bool: ...
    def AddValueChanged(self, component: typing.Any, handler: EventHandler) -> None: ...
    @abc.abstractmethod
    def CanResetValue(self, component: typing.Any) -> bool: ...
    def Equals(self, obj: typing.Any) -> bool: ...
    def GetEditor(self, editorBaseType: typing.Type[typing.Any]) -> typing.Any: ...
    def GetHashCode(self) -> int: ...
    @abc.abstractmethod
    def GetValue(self, component: typing.Any) -> typing.Any: ...
    def RemoveValueChanged(self, component: typing.Any, handler: EventHandler) -> None: ...
    @abc.abstractmethod
    def ResetValue(self, component: typing.Any) -> None: ...
    @abc.abstractmethod
    def SetValue(self, component: typing.Any, value: typing.Any) -> None: ...
    @abc.abstractmethod
    def ShouldSerializeValue(self, component: typing.Any) -> bool: ...
    # Skipped GetChildProperties due to it being static, abstract and generic.

    GetChildProperties : GetChildProperties_MethodGroup
    class GetChildProperties_MethodGroup:
        @typing.overload
        def __call__(self) -> PropertyDescriptorCollection:...
        @typing.overload
        def __call__(self, filter: Array_1[Attribute]) -> PropertyDescriptorCollection:...
        @typing.overload
        def __call__(self, instance: typing.Any) -> PropertyDescriptorCollection:...
        @typing.overload
        def __call__(self, instance: typing.Any, filter: Array_1[Attribute]) -> PropertyDescriptorCollection:...



class PropertyDescriptorCollection(IDictionary, IList):
    @typing.overload
    def __init__(self, properties: Array_1[PropertyDescriptor]) -> None: ...
    @typing.overload
    def __init__(self, properties: Array_1[PropertyDescriptor], readOnly: bool) -> None: ...
    Empty : PropertyDescriptorCollection
    @property
    def Count(self) -> int: ...
    @Count.setter
    def Count(self, value: int) -> int: ...
    @property
    def Item(self) -> PropertyDescriptor: ...
    @property
    def Item(self) -> PropertyDescriptor: ...
    def Add(self, value: PropertyDescriptor) -> int: ...
    def Clear(self) -> None: ...
    def Contains(self, value: PropertyDescriptor) -> bool: ...
    def CopyTo(self, array: Array, index: int) -> None: ...
    def Find(self, name: str, ignoreCase: bool) -> PropertyDescriptor: ...
    def GetEnumerator(self) -> IEnumerator: ...
    def IndexOf(self, value: PropertyDescriptor) -> int: ...
    def Insert(self, index: int, value: PropertyDescriptor) -> None: ...
    def Remove(self, value: PropertyDescriptor) -> None: ...
    def RemoveAt(self, index: int) -> None: ...
    # Skipped Sort due to it being static, abstract and generic.

    Sort : Sort_MethodGroup
    class Sort_MethodGroup:
        @typing.overload
        def __call__(self) -> PropertyDescriptorCollection:...
        @typing.overload
        def __call__(self, names: Array_1[str]) -> PropertyDescriptorCollection:...
        @typing.overload
        def __call__(self, comparer: IComparer) -> PropertyDescriptorCollection:...
        @typing.overload
        def __call__(self, names: Array_1[str], comparer: IComparer) -> PropertyDescriptorCollection:...



class TypeConverter:
    def __init__(self) -> None: ...
    # Skipped CanConvertFrom due to it being static, abstract and generic.

    CanConvertFrom : CanConvertFrom_MethodGroup
    class CanConvertFrom_MethodGroup:
        @typing.overload
        def __call__(self, sourceType: typing.Type[typing.Any]) -> bool:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext, sourceType: typing.Type[typing.Any]) -> bool:...

    # Skipped CanConvertTo due to it being static, abstract and generic.

    CanConvertTo : CanConvertTo_MethodGroup
    class CanConvertTo_MethodGroup:
        @typing.overload
        def __call__(self, destinationType: typing.Type[typing.Any]) -> bool:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext, destinationType: typing.Type[typing.Any]) -> bool:...

    # Skipped ConvertFrom due to it being static, abstract and generic.

    ConvertFrom : ConvertFrom_MethodGroup
    class ConvertFrom_MethodGroup:
        @typing.overload
        def __call__(self, value: typing.Any) -> typing.Any:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any) -> typing.Any:...

    # Skipped ConvertFromInvariantString due to it being static, abstract and generic.

    ConvertFromInvariantString : ConvertFromInvariantString_MethodGroup
    class ConvertFromInvariantString_MethodGroup:
        @typing.overload
        def __call__(self, text: str) -> typing.Any:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext, text: str) -> typing.Any:...

    # Skipped ConvertFromString due to it being static, abstract and generic.

    ConvertFromString : ConvertFromString_MethodGroup
    class ConvertFromString_MethodGroup:
        @typing.overload
        def __call__(self, text: str) -> typing.Any:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext, text: str) -> typing.Any:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext, culture: CultureInfo, text: str) -> typing.Any:...

    # Skipped ConvertTo due to it being static, abstract and generic.

    ConvertTo : ConvertTo_MethodGroup
    class ConvertTo_MethodGroup:
        @typing.overload
        def __call__(self, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any:...

    # Skipped ConvertToInvariantString due to it being static, abstract and generic.

    ConvertToInvariantString : ConvertToInvariantString_MethodGroup
    class ConvertToInvariantString_MethodGroup:
        @typing.overload
        def __call__(self, value: typing.Any) -> str:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext, value: typing.Any) -> str:...

    # Skipped ConvertToString due to it being static, abstract and generic.

    ConvertToString : ConvertToString_MethodGroup
    class ConvertToString_MethodGroup:
        @typing.overload
        def __call__(self, value: typing.Any) -> str:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext, value: typing.Any) -> str:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any) -> str:...

    # Skipped CreateInstance due to it being static, abstract and generic.

    CreateInstance : CreateInstance_MethodGroup
    class CreateInstance_MethodGroup:
        @typing.overload
        def __call__(self, propertyValues: IDictionary) -> typing.Any:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext, propertyValues: IDictionary) -> typing.Any:...

    # Skipped GetCreateInstanceSupported due to it being static, abstract and generic.

    GetCreateInstanceSupported : GetCreateInstanceSupported_MethodGroup
    class GetCreateInstanceSupported_MethodGroup:
        @typing.overload
        def __call__(self) -> bool:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext) -> bool:...

    # Skipped GetProperties due to it being static, abstract and generic.

    GetProperties : GetProperties_MethodGroup
    class GetProperties_MethodGroup:
        @typing.overload
        def __call__(self, value: typing.Any) -> PropertyDescriptorCollection:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext, value: typing.Any) -> PropertyDescriptorCollection:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext, value: typing.Any, attributes: Array_1[Attribute]) -> PropertyDescriptorCollection:...

    # Skipped GetPropertiesSupported due to it being static, abstract and generic.

    GetPropertiesSupported : GetPropertiesSupported_MethodGroup
    class GetPropertiesSupported_MethodGroup:
        @typing.overload
        def __call__(self) -> bool:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext) -> bool:...

    # Skipped GetStandardValues due to it being static, abstract and generic.

    GetStandardValues : GetStandardValues_MethodGroup
    class GetStandardValues_MethodGroup:
        @typing.overload
        def __call__(self) -> ICollection:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext) -> TypeConverter.StandardValuesCollection:...

    # Skipped GetStandardValuesExclusive due to it being static, abstract and generic.

    GetStandardValuesExclusive : GetStandardValuesExclusive_MethodGroup
    class GetStandardValuesExclusive_MethodGroup:
        @typing.overload
        def __call__(self) -> bool:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext) -> bool:...

    # Skipped GetStandardValuesSupported due to it being static, abstract and generic.

    GetStandardValuesSupported : GetStandardValuesSupported_MethodGroup
    class GetStandardValuesSupported_MethodGroup:
        @typing.overload
        def __call__(self) -> bool:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext) -> bool:...

    # Skipped IsValid due to it being static, abstract and generic.

    IsValid : IsValid_MethodGroup
    class IsValid_MethodGroup:
        @typing.overload
        def __call__(self, value: typing.Any) -> bool:...
        @typing.overload
        def __call__(self, context: ITypeDescriptorContext, value: typing.Any) -> bool:...


    class StandardValuesCollection(ICollection):
        def __init__(self, values: ICollection) -> None: ...
        @property
        def Count(self) -> int: ...
        @property
        def Item(self) -> typing.Any: ...
        def CopyTo(self, array: Array, index: int) -> None: ...
        def GetEnumerator(self) -> IEnumerator: ...



class Win32Exception(ExternalException):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, error: int) -> None: ...
    @typing.overload
    def __init__(self, error: int, message: str) -> None: ...
    @typing.overload
    def __init__(self, message: str) -> None: ...
    @typing.overload
    def __init__(self, message: str, innerException: Exception) -> None: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def ErrorCode(self) -> int: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def NativeErrorCode(self) -> int: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...
    def GetObjectData(self, info: SerializationInfo, context: StreamingContext) -> None: ...
    def ToString(self) -> str: ...




import typing, clr, abc
from SIMULTAN.Data.SimNetworks import INetwork
from SIMULTAN.Data import IReferenceLocation, GeometricReference, SimManagedCollection_1, IReference
from SIMULTAN.Data.SimMath import SimPoint
from SIMULTAN.Data.Users import SimUserRole
from System.Collections.ObjectModel import ObservableCollection_1
from System.Collections.Generic import List_1, IDictionary_2, Dictionary_2, IEnumerable_1
from SIMULTAN.Utils.Collections import ObservableDictionary_2
from SIMULTAN.Data.Components import SimComponentInstance, SimComponent, SimInstanceSize
from System import Guid, MulticastDelegate, IAsyncResult, AsyncCallback, DateTime, IComparable, ValueTuple_3
from System.Globalization import CultureInfo
from System.Reflection import MethodInfo
from SIMULTAN.Projects import ProjectData
from System.ComponentModel import INotifyPropertyChanged

class SimFlowNetwork(SimFlowNetworkNode, INetwork):
    def __init__(self, _location: IReferenceLocation, _position: SimPoint, _name: str, _description: str, _manager: SimUserRole) -> None: ...
    @property
    def CalculationRules(self) -> ObservableCollection_1[SimFlowNetworkCalcRule]: ...
    @CalculationRules.setter
    def CalculationRules(self, value: ObservableCollection_1[SimFlowNetworkCalcRule]) -> ObservableCollection_1[SimFlowNetworkCalcRule]: ...
    @property
    def Children(self) -> List_1[typing.Any]: ...
    @property
    def ConnectionToParentEntryNode(self) -> SimFlowNetworkNode: ...
    @property
    def ConnectionToParentExitNode(self) -> SimFlowNetworkNode: ...
    @property
    def ContainedEdges(self) -> ObservableDictionary_2[int, SimFlowNetworkEdge]: ...
    @ContainedEdges.setter
    def ContainedEdges(self, value: ObservableDictionary_2[int, SimFlowNetworkEdge]) -> ObservableDictionary_2[int, SimFlowNetworkEdge]: ...
    @property
    def ContainedFlowNetworks(self) -> ObservableDictionary_2[int, SimFlowNetwork]: ...
    @ContainedFlowNetworks.setter
    def ContainedFlowNetworks(self, value: ObservableDictionary_2[int, SimFlowNetwork]) -> ObservableDictionary_2[int, SimFlowNetwork]: ...
    @property
    def ContainedNodes(self) -> ObservableDictionary_2[int, SimFlowNetworkNode]: ...
    @ContainedNodes.setter
    def ContainedNodes(self, value: ObservableDictionary_2[int, SimFlowNetworkNode]) -> ObservableDictionary_2[int, SimFlowNetworkNode]: ...
    @property
    def Content(self) -> SimComponentInstance: ...
    @Content.setter
    def Content(self, value: SimComponentInstance) -> SimComponentInstance: ...
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def Edges_In(self) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @Edges_In.setter
    def Edges_In(self, value: ObservableCollection_1[SimFlowNetworkEdge]) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @property
    def Edges_In_Nested(self) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @Edges_In_Nested.setter
    def Edges_In_Nested(self, value: ObservableCollection_1[SimFlowNetworkEdge]) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @property
    def Edges_Out(self) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @Edges_Out.setter
    def Edges_Out(self, value: ObservableCollection_1[SimFlowNetworkEdge]) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @property
    def Edges_Out_Nested(self) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @Edges_Out_Nested.setter
    def Edges_Out_Nested(self, value: ObservableCollection_1[SimFlowNetworkEdge]) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def HasParent(self) -> bool: ...
    @property
    def ID(self) -> SimObjectId: ...
    @ID.setter
    def ID(self, value: SimObjectId) -> SimObjectId: ...
    @property
    def IndexOfGeometricRepFile(self) -> int: ...
    @IndexOfGeometricRepFile.setter
    def IndexOfGeometricRepFile(self, value: int) -> int: ...
    @property
    def IsDirected(self) -> bool: ...
    @IsDirected.setter
    def IsDirected(self, value: bool) -> bool: ...
    @property
    def IsInConsistentState(self) -> bool: ...
    @IsInConsistentState.setter
    def IsInConsistentState(self, value: bool) -> bool: ...
    @property
    def IsValid(self) -> bool: ...
    @IsValid.setter
    def IsValid(self, value: bool) -> bool: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def Manager(self) -> SimUserRole: ...
    @Manager.setter
    def Manager(self, value: SimUserRole) -> SimUserRole: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def Network(self) -> SimFlowNetwork: ...
    @Network.setter
    def Network(self, value: SimFlowNetwork) -> SimFlowNetwork: ...
    @property
    def NodeEnd_ID(self) -> int: ...
    @NodeEnd_ID.setter
    def NodeEnd_ID(self, value: int) -> int: ...
    @property
    def NodeStart_ID(self) -> int: ...
    @NodeStart_ID.setter
    def NodeStart_ID(self, value: int) -> int: ...
    @property
    def Parent(self) -> SimFlowNetworkElement: ...
    @Parent.setter
    def Parent(self, value: SimFlowNetworkElement) -> SimFlowNetworkElement: ...
    @property
    def ParentNetwork(self) -> SimFlowNetwork: ...
    @ParentNetwork.setter
    def ParentNetwork(self, value: SimFlowNetwork) -> SimFlowNetwork: ...
    @property
    def Position(self) -> SimPoint: ...
    @Position.setter
    def Position(self, value: SimPoint) -> SimPoint: ...
    @property
    def RepresentationReference(self) -> GeometricReference: ...
    @RepresentationReference.setter
    def RepresentationReference(self, value: GeometricReference) -> GeometricReference: ...
    def AddEdge(self, _start: SimFlowNetworkNode, _end: SimFlowNetworkNode) -> int: ...
    def AddNode(self, _pos: SimPoint) -> int: ...
    def CalculateAllFlows(self, _in_flow_dir: bool, culture: CultureInfo) -> None: ...
    def CalculateFlowStep(self, _sorted_nodes: List_1[SimFlowNetworkNode], _in_flow_dir: bool, _step_index: int, _show_only_changes: bool, _current_node: clr.Reference[SimFlowNetworkNode], culture: CultureInfo) -> str: ...
    def GetAllContainersOf(self, _comp: SimComponent) -> List_1[SimFlowNetworkElement]: ...
    def GetAllContent(self) -> List_1[SimComponent]: ...
    def GetNestedEdges(self) -> List_1[SimFlowNetworkEdge]: ...
    def GetNestedNodes(self) -> List_1[SimFlowNetworkNode]: ...
    def GetUniqueParamNamesInContent(self, culture: CultureInfo) -> List_1[str]: ...
    def OnEdgeRedirected(self, redirectedEdge: SimFlowNetworkEdge) -> None: ...
    def OnElementAdded(self, addedElement: SimFlowNetworkElement) -> None: ...
    def OnElementDeleted(self, deletedElement: SimFlowNetworkElement) -> None: ...
    def OnElementTopologyChanged(self, oldElement: SimFlowNetworkElement, newElement: SimFlowNetworkElement, changedElements: List_1[SimFlowNetworkElement]) -> None: ...
    def PrepareToCalculateFlowStepByStep(self, _in_flow_dir: bool) -> List_1[SimFlowNetworkNode]: ...
    def PropagateCalculationRulesToAllInstances(self, _source: SimFlowNetworkNode) -> None: ...
    def PropagateSizeToInstances(self, _source: SimFlowNetworkElement, _targets: List_1[SimFlowNetworkElement]) -> None: ...
    def RedirectEdge(self, _edge: SimFlowNetworkEdge, _rerout_start: bool, _to_node: SimFlowNetworkNode) -> bool: ...
    def RemoveEdge(self, _edge: SimFlowNetworkEdge) -> bool: ...
    def RemoveNodeOrNetwork(self, _node: SimFlowNetworkNode) -> bool: ...
    def ResetAllContentInstances(self, _offse_parent: SimPoint) -> None: ...
    def ToString(self) -> str: ...
    def UpdateChildrenContainer(self) -> None: ...

    class EdgeRedirectedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, redirectedEdge: SimFlowNetworkEdge, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, redirectedEdge: SimFlowNetworkEdge) -> None: ...


    class ElementAddedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, addedElement: SimFlowNetworkElement, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, addedElement: SimFlowNetworkElement) -> None: ...


    class ElementDeletedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, deletedElement: SimFlowNetworkElement, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, deletedElement: SimFlowNetworkElement) -> None: ...


    class ElementTopologyChangedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, oldElement: SimFlowNetworkElement, newElement: SimFlowNetworkElement, changedElements: List_1[SimFlowNetworkElement], callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, oldElement: SimFlowNetworkElement, newElement: SimFlowNetworkElement, changedElements: List_1[SimFlowNetworkElement]) -> None: ...



class SimFlowNetworkCalcDirection(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Forward : SimFlowNetworkCalcDirection # 0
    Backward : SimFlowNetworkCalcDirection # 1


class SimFlowNetworkCalcRule:
    @property
    def Direction(self) -> SimFlowNetworkCalcDirection: ...
    @Direction.setter
    def Direction(self, value: SimFlowNetworkCalcDirection) -> SimFlowNetworkCalcDirection: ...
    @property
    def Operator(self) -> SimFlowNetworkOperator: ...
    @Operator.setter
    def Operator(self, value: SimFlowNetworkOperator) -> SimFlowNetworkOperator: ...
    @property
    def Suffix_Operands(self) -> str: ...
    @Suffix_Operands.setter
    def Suffix_Operands(self, value: str) -> str: ...
    @property
    def Suffix_Result(self) -> str: ...
    @Suffix_Result.setter
    def Suffix_Result(self, value: str) -> str: ...
    def Calculate(self, _v1: float, _v2: float) -> float: ...
    @staticmethod
    def OperatorToString(_op: SimFlowNetworkOperator) -> str: ...
    @staticmethod
    def StringToOperator(_op_as_str: str) -> SimFlowNetworkOperator: ...


class SimFlowNetworkEdge(SimFlowNetworkElement):
    @property
    def Content(self) -> SimComponentInstance: ...
    @Content.setter
    def Content(self, value: SimComponentInstance) -> SimComponentInstance: ...
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def End(self) -> SimFlowNetworkNode: ...
    @End.setter
    def End(self, value: SimFlowNetworkNode) -> SimFlowNetworkNode: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def ID(self) -> SimObjectId: ...
    @ID.setter
    def ID(self, value: SimObjectId) -> SimObjectId: ...
    @property
    def IsValid(self) -> bool: ...
    @IsValid.setter
    def IsValid(self, value: bool) -> bool: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def Network(self) -> SimFlowNetwork: ...
    @Network.setter
    def Network(self, value: SimFlowNetwork) -> SimFlowNetwork: ...
    @property
    def Parent(self) -> SimFlowNetworkElement: ...
    @Parent.setter
    def Parent(self, value: SimFlowNetworkElement) -> SimFlowNetworkElement: ...
    @property
    def RepresentationReference(self) -> GeometricReference: ...
    @RepresentationReference.setter
    def RepresentationReference(self, value: GeometricReference) -> GeometricReference: ...
    @property
    def Start(self) -> SimFlowNetworkNode: ...
    @Start.setter
    def Start(self, value: SimFlowNetworkNode) -> SimFlowNetworkNode: ...
    def RestoreReferences(self, nodes: IDictionary_2[int, SimFlowNetworkNode], networks: IDictionary_2[int, SimFlowNetwork]) -> None: ...
    def ToString(self) -> str: ...


class SimFlowNetworkElement(SimObject):
    @property
    def Content(self) -> SimComponentInstance: ...
    @Content.setter
    def Content(self, value: SimComponentInstance) -> SimComponentInstance: ...
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def ID(self) -> SimObjectId: ...
    @ID.setter
    def ID(self, value: SimObjectId) -> SimObjectId: ...
    @property
    def IsValid(self) -> bool: ...
    @IsValid.setter
    def IsValid(self, value: bool) -> bool: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def Network(self) -> SimFlowNetwork: ...
    @Network.setter
    def Network(self, value: SimFlowNetwork) -> SimFlowNetwork: ...
    @property
    def Parent(self) -> SimFlowNetworkElement: ...
    @Parent.setter
    def Parent(self, value: SimFlowNetworkElement) -> SimFlowNetworkElement: ...
    @property
    def RepresentationReference(self) -> GeometricReference: ...
    @RepresentationReference.setter
    def RepresentationReference(self, value: GeometricReference) -> GeometricReference: ...
    def GetBoundInstanceRealizedStatus(self) -> bool: ...
    def GetInstanceSize(self) -> SimInstanceSize: ...
    def InstanceHasPath(self) -> bool: ...
    def InstanceHasValidPath(self) -> bool: ...


class SimFlowNetworkNode(SimFlowNetworkElement):
    @property
    def CalculationRules(self) -> ObservableCollection_1[SimFlowNetworkCalcRule]: ...
    @CalculationRules.setter
    def CalculationRules(self, value: ObservableCollection_1[SimFlowNetworkCalcRule]) -> ObservableCollection_1[SimFlowNetworkCalcRule]: ...
    @property
    def Content(self) -> SimComponentInstance: ...
    @Content.setter
    def Content(self, value: SimComponentInstance) -> SimComponentInstance: ...
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def Edges_In(self) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @Edges_In.setter
    def Edges_In(self, value: ObservableCollection_1[SimFlowNetworkEdge]) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @property
    def Edges_In_Nested(self) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @Edges_In_Nested.setter
    def Edges_In_Nested(self, value: ObservableCollection_1[SimFlowNetworkEdge]) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @property
    def Edges_Out(self) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @Edges_Out.setter
    def Edges_Out(self, value: ObservableCollection_1[SimFlowNetworkEdge]) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @property
    def Edges_Out_Nested(self) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @Edges_Out_Nested.setter
    def Edges_Out_Nested(self, value: ObservableCollection_1[SimFlowNetworkEdge]) -> ObservableCollection_1[SimFlowNetworkEdge]: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def ID(self) -> SimObjectId: ...
    @ID.setter
    def ID(self, value: SimObjectId) -> SimObjectId: ...
    @property
    def IsValid(self) -> bool: ...
    @IsValid.setter
    def IsValid(self, value: bool) -> bool: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def Network(self) -> SimFlowNetwork: ...
    @Network.setter
    def Network(self, value: SimFlowNetwork) -> SimFlowNetwork: ...
    @property
    def Parent(self) -> SimFlowNetworkElement: ...
    @Parent.setter
    def Parent(self, value: SimFlowNetworkElement) -> SimFlowNetworkElement: ...
    @property
    def Position(self) -> SimPoint: ...
    @Position.setter
    def Position(self, value: SimPoint) -> SimPoint: ...
    @property
    def RepresentationReference(self) -> GeometricReference: ...
    @RepresentationReference.setter
    def RepresentationReference(self, value: GeometricReference) -> GeometricReference: ...
    def ToString(self) -> str: ...


class SimFlowNetworkOperator(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Addition : SimFlowNetworkOperator # 0
    Subtraction : SimFlowNetworkOperator # 1
    Multiplication : SimFlowNetworkOperator # 2
    Division : SimFlowNetworkOperator # 3
    Minimum : SimFlowNetworkOperator # 4
    Maximum : SimFlowNetworkOperator # 5
    Assignment : SimFlowNetworkOperator # 6


class SimNetworkFactory(SimManagedCollection_1[int]):
    def __init__(self, owner: ProjectData) -> None: ...
    @property
    def CalledFromLocation(self) -> IReferenceLocation: ...
    @property
    def Count(self) -> int: ...
    @property
    def HasChanges(self) -> bool: ...
    @property
    def Item(self) -> int: ...
    @Item.setter
    def Item(self, value: int) -> int: ...
    @property
    def LastChange(self) -> DateTime: ...
    @property
    def NetworkRecord(self) -> ObservableCollection_1[SimFlowNetwork]: ...
    @NetworkRecord.setter
    def NetworkRecord(self, value: ObservableCollection_1[SimFlowNetwork]) -> ObservableCollection_1[SimFlowNetwork]: ...
    @property
    def ProjectData(self) -> ProjectData: ...
    def CreateEmptyNetwork(self, name: str, _user: SimUserRole) -> SimFlowNetwork: ...
    def GetAllNetworkElements(self) -> Dictionary_2[SimObjectId, SimFlowNetworkElement]: ...
    def OnNetworkAdded(self, addedNetworks: IEnumerable_1[SimFlowNetwork]) -> None: ...
    def OnNetworkDeleted(self, deletedNetworks: IEnumerable_1[SimFlowNetwork]) -> None: ...
    def RemoveReferencesToGeometryWithinRecord(self) -> None: ...

    class NetworkAddedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, addedNetworks: IEnumerable_1[SimFlowNetwork], callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, addedNetworks: IEnumerable_1[SimFlowNetwork]) -> None: ...


    class NetworkDeletedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, deletedNetworks: IEnumerable_1[SimFlowNetwork], callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, deletedNetworks: IEnumerable_1[SimFlowNetwork]) -> None: ...



class SimObject(IReference, INotifyPropertyChanged, abc.ABC):
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def ID(self) -> SimObjectId: ...
    @ID.setter
    def ID(self, value: SimObjectId) -> SimObjectId: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...


class SimObjectId(IComparable, INotifyPropertyChanged):
    @typing.overload
    def __init__(self, globalId: Guid, localId: int) -> None: ...
    @typing.overload
    def __init__(self, globalLocation: IReferenceLocation, localId: int) -> None: ...
    @typing.overload
    def __init__(self, localId: int) -> None: ...
    Empty : SimObjectId
    @property
    def GlobalId(self) -> Guid: ...
    @GlobalId.setter
    def GlobalId(self, value: Guid) -> Guid: ...
    @property
    def GlobalLocation(self) -> IReferenceLocation: ...
    @GlobalLocation.setter
    def GlobalLocation(self, value: IReferenceLocation) -> IReferenceLocation: ...
    @property
    def LocalId(self) -> int: ...
    @LocalId.setter
    def LocalId(self, value: int) -> int: ...
    def CompareTo(self, obj: typing.Any) -> int: ...
    def Equals(self, obj: typing.Any) -> bool: ...
    @staticmethod
    def FromString(simObjectString: str) -> ValueTuple_3[Guid, int, int]: ...
    def GetHashCode(self) -> int: ...
    def __eq__(self, pid1: SimObjectId, pid2: SimObjectId) -> bool: ...
    def __ne__(self, pid1: SimObjectId, pid2: SimObjectId) -> bool: ...
    def ToString(self) -> str: ...




import typing, clr, abc
from System import ReadOnlySpan_1, Half, Int128, UInt128, UIntPtr, Span_1

class BinaryPrimitives(abc.ABC):
    @staticmethod
    def ReadDoubleBigEndian(source: ReadOnlySpan_1[int]) -> float: ...
    @staticmethod
    def ReadDoubleLittleEndian(source: ReadOnlySpan_1[int]) -> float: ...
    @staticmethod
    def ReadHalfBigEndian(source: ReadOnlySpan_1[int]) -> Half: ...
    @staticmethod
    def ReadHalfLittleEndian(source: ReadOnlySpan_1[int]) -> Half: ...
    @staticmethod
    def ReadInt128BigEndian(source: ReadOnlySpan_1[int]) -> Int128: ...
    @staticmethod
    def ReadInt128LittleEndian(source: ReadOnlySpan_1[int]) -> Int128: ...
    @staticmethod
    def ReadInt16BigEndian(source: ReadOnlySpan_1[int]) -> int: ...
    @staticmethod
    def ReadInt16LittleEndian(source: ReadOnlySpan_1[int]) -> int: ...
    @staticmethod
    def ReadInt32BigEndian(source: ReadOnlySpan_1[int]) -> int: ...
    @staticmethod
    def ReadInt32LittleEndian(source: ReadOnlySpan_1[int]) -> int: ...
    @staticmethod
    def ReadInt64BigEndian(source: ReadOnlySpan_1[int]) -> int: ...
    @staticmethod
    def ReadInt64LittleEndian(source: ReadOnlySpan_1[int]) -> int: ...
    @staticmethod
    def ReadIntPtrBigEndian(source: ReadOnlySpan_1[int]) -> int: ...
    @staticmethod
    def ReadIntPtrLittleEndian(source: ReadOnlySpan_1[int]) -> int: ...
    @staticmethod
    def ReadSingleBigEndian(source: ReadOnlySpan_1[int]) -> float: ...
    @staticmethod
    def ReadSingleLittleEndian(source: ReadOnlySpan_1[int]) -> float: ...
    @staticmethod
    def ReadUInt128BigEndian(source: ReadOnlySpan_1[int]) -> UInt128: ...
    @staticmethod
    def ReadUInt128LittleEndian(source: ReadOnlySpan_1[int]) -> UInt128: ...
    @staticmethod
    def ReadUInt16BigEndian(source: ReadOnlySpan_1[int]) -> int: ...
    @staticmethod
    def ReadUInt16LittleEndian(source: ReadOnlySpan_1[int]) -> int: ...
    @staticmethod
    def ReadUInt32BigEndian(source: ReadOnlySpan_1[int]) -> int: ...
    @staticmethod
    def ReadUInt32LittleEndian(source: ReadOnlySpan_1[int]) -> int: ...
    @staticmethod
    def ReadUInt64BigEndian(source: ReadOnlySpan_1[int]) -> int: ...
    @staticmethod
    def ReadUInt64LittleEndian(source: ReadOnlySpan_1[int]) -> int: ...
    @staticmethod
    def ReadUIntPtrBigEndian(source: ReadOnlySpan_1[int]) -> UIntPtr: ...
    @staticmethod
    def ReadUIntPtrLittleEndian(source: ReadOnlySpan_1[int]) -> UIntPtr: ...
    @staticmethod
    def TryReadDoubleBigEndian(source: ReadOnlySpan_1[int], value: clr.Reference[float]) -> bool: ...
    @staticmethod
    def TryReadDoubleLittleEndian(source: ReadOnlySpan_1[int], value: clr.Reference[float]) -> bool: ...
    @staticmethod
    def TryReadHalfBigEndian(source: ReadOnlySpan_1[int], value: clr.Reference[Half]) -> bool: ...
    @staticmethod
    def TryReadHalfLittleEndian(source: ReadOnlySpan_1[int], value: clr.Reference[Half]) -> bool: ...
    @staticmethod
    def TryReadInt128BigEndian(source: ReadOnlySpan_1[int], value: clr.Reference[Int128]) -> bool: ...
    @staticmethod
    def TryReadInt128LittleEndian(source: ReadOnlySpan_1[int], value: clr.Reference[Int128]) -> bool: ...
    @staticmethod
    def TryReadInt16BigEndian(source: ReadOnlySpan_1[int], value: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryReadInt16LittleEndian(source: ReadOnlySpan_1[int], value: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryReadInt32BigEndian(source: ReadOnlySpan_1[int], value: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryReadInt32LittleEndian(source: ReadOnlySpan_1[int], value: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryReadInt64BigEndian(source: ReadOnlySpan_1[int], value: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryReadInt64LittleEndian(source: ReadOnlySpan_1[int], value: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryReadIntPtrBigEndian(source: ReadOnlySpan_1[int], value: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryReadIntPtrLittleEndian(source: ReadOnlySpan_1[int], value: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryReadSingleBigEndian(source: ReadOnlySpan_1[int], value: clr.Reference[float]) -> bool: ...
    @staticmethod
    def TryReadSingleLittleEndian(source: ReadOnlySpan_1[int], value: clr.Reference[float]) -> bool: ...
    @staticmethod
    def TryReadUInt128BigEndian(source: ReadOnlySpan_1[int], value: clr.Reference[UInt128]) -> bool: ...
    @staticmethod
    def TryReadUInt128LittleEndian(source: ReadOnlySpan_1[int], value: clr.Reference[UInt128]) -> bool: ...
    @staticmethod
    def TryReadUInt16BigEndian(source: ReadOnlySpan_1[int], value: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryReadUInt16LittleEndian(source: ReadOnlySpan_1[int], value: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryReadUInt32BigEndian(source: ReadOnlySpan_1[int], value: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryReadUInt32LittleEndian(source: ReadOnlySpan_1[int], value: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryReadUInt64BigEndian(source: ReadOnlySpan_1[int], value: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryReadUInt64LittleEndian(source: ReadOnlySpan_1[int], value: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryReadUIntPtrBigEndian(source: ReadOnlySpan_1[int], value: clr.Reference[UIntPtr]) -> bool: ...
    @staticmethod
    def TryReadUIntPtrLittleEndian(source: ReadOnlySpan_1[int], value: clr.Reference[UIntPtr]) -> bool: ...
    @staticmethod
    def TryWriteDoubleBigEndian(destination: Span_1[int], value: float) -> bool: ...
    @staticmethod
    def TryWriteDoubleLittleEndian(destination: Span_1[int], value: float) -> bool: ...
    @staticmethod
    def TryWriteHalfBigEndian(destination: Span_1[int], value: Half) -> bool: ...
    @staticmethod
    def TryWriteHalfLittleEndian(destination: Span_1[int], value: Half) -> bool: ...
    @staticmethod
    def TryWriteInt128BigEndian(destination: Span_1[int], value: Int128) -> bool: ...
    @staticmethod
    def TryWriteInt128LittleEndian(destination: Span_1[int], value: Int128) -> bool: ...
    @staticmethod
    def TryWriteInt16BigEndian(destination: Span_1[int], value: int) -> bool: ...
    @staticmethod
    def TryWriteInt16LittleEndian(destination: Span_1[int], value: int) -> bool: ...
    @staticmethod
    def TryWriteInt32BigEndian(destination: Span_1[int], value: int) -> bool: ...
    @staticmethod
    def TryWriteInt32LittleEndian(destination: Span_1[int], value: int) -> bool: ...
    @staticmethod
    def TryWriteInt64BigEndian(destination: Span_1[int], value: int) -> bool: ...
    @staticmethod
    def TryWriteInt64LittleEndian(destination: Span_1[int], value: int) -> bool: ...
    @staticmethod
    def TryWriteIntPtrBigEndian(destination: Span_1[int], value: int) -> bool: ...
    @staticmethod
    def TryWriteIntPtrLittleEndian(destination: Span_1[int], value: int) -> bool: ...
    @staticmethod
    def TryWriteSingleBigEndian(destination: Span_1[int], value: float) -> bool: ...
    @staticmethod
    def TryWriteSingleLittleEndian(destination: Span_1[int], value: float) -> bool: ...
    @staticmethod
    def TryWriteUInt128BigEndian(destination: Span_1[int], value: UInt128) -> bool: ...
    @staticmethod
    def TryWriteUInt128LittleEndian(destination: Span_1[int], value: UInt128) -> bool: ...
    @staticmethod
    def TryWriteUInt16BigEndian(destination: Span_1[int], value: int) -> bool: ...
    @staticmethod
    def TryWriteUInt16LittleEndian(destination: Span_1[int], value: int) -> bool: ...
    @staticmethod
    def TryWriteUInt32BigEndian(destination: Span_1[int], value: int) -> bool: ...
    @staticmethod
    def TryWriteUInt32LittleEndian(destination: Span_1[int], value: int) -> bool: ...
    @staticmethod
    def TryWriteUInt64BigEndian(destination: Span_1[int], value: int) -> bool: ...
    @staticmethod
    def TryWriteUInt64LittleEndian(destination: Span_1[int], value: int) -> bool: ...
    @staticmethod
    def TryWriteUIntPtrBigEndian(destination: Span_1[int], value: UIntPtr) -> bool: ...
    @staticmethod
    def TryWriteUIntPtrLittleEndian(destination: Span_1[int], value: UIntPtr) -> bool: ...
    @staticmethod
    def WriteDoubleBigEndian(destination: Span_1[int], value: float) -> None: ...
    @staticmethod
    def WriteDoubleLittleEndian(destination: Span_1[int], value: float) -> None: ...
    @staticmethod
    def WriteHalfBigEndian(destination: Span_1[int], value: Half) -> None: ...
    @staticmethod
    def WriteHalfLittleEndian(destination: Span_1[int], value: Half) -> None: ...
    @staticmethod
    def WriteInt128BigEndian(destination: Span_1[int], value: Int128) -> None: ...
    @staticmethod
    def WriteInt128LittleEndian(destination: Span_1[int], value: Int128) -> None: ...
    @staticmethod
    def WriteInt16BigEndian(destination: Span_1[int], value: int) -> None: ...
    @staticmethod
    def WriteInt16LittleEndian(destination: Span_1[int], value: int) -> None: ...
    @staticmethod
    def WriteInt32BigEndian(destination: Span_1[int], value: int) -> None: ...
    @staticmethod
    def WriteInt32LittleEndian(destination: Span_1[int], value: int) -> None: ...
    @staticmethod
    def WriteInt64BigEndian(destination: Span_1[int], value: int) -> None: ...
    @staticmethod
    def WriteInt64LittleEndian(destination: Span_1[int], value: int) -> None: ...
    @staticmethod
    def WriteIntPtrBigEndian(destination: Span_1[int], value: int) -> None: ...
    @staticmethod
    def WriteIntPtrLittleEndian(destination: Span_1[int], value: int) -> None: ...
    @staticmethod
    def WriteSingleBigEndian(destination: Span_1[int], value: float) -> None: ...
    @staticmethod
    def WriteSingleLittleEndian(destination: Span_1[int], value: float) -> None: ...
    @staticmethod
    def WriteUInt128BigEndian(destination: Span_1[int], value: UInt128) -> None: ...
    @staticmethod
    def WriteUInt128LittleEndian(destination: Span_1[int], value: UInt128) -> None: ...
    @staticmethod
    def WriteUInt16BigEndian(destination: Span_1[int], value: int) -> None: ...
    @staticmethod
    def WriteUInt16LittleEndian(destination: Span_1[int], value: int) -> None: ...
    @staticmethod
    def WriteUInt32BigEndian(destination: Span_1[int], value: int) -> None: ...
    @staticmethod
    def WriteUInt32LittleEndian(destination: Span_1[int], value: int) -> None: ...
    @staticmethod
    def WriteUInt64BigEndian(destination: Span_1[int], value: int) -> None: ...
    @staticmethod
    def WriteUInt64LittleEndian(destination: Span_1[int], value: int) -> None: ...
    @staticmethod
    def WriteUIntPtrBigEndian(destination: Span_1[int], value: UIntPtr) -> None: ...
    @staticmethod
    def WriteUIntPtrLittleEndian(destination: Span_1[int], value: UIntPtr) -> None: ...
    # Skipped ReverseEndianness due to it being static, abstract and generic.

    ReverseEndianness : ReverseEndianness_MethodGroup
    class ReverseEndianness_MethodGroup:
        @typing.overload
        def __call__(self, value: int) -> int:...
        # Method ReverseEndianness(value : Int16) was skipped since it collides with above method
        # Method ReverseEndianness(value : Int32) was skipped since it collides with above method
        # Method ReverseEndianness(value : Int64) was skipped since it collides with above method
        @typing.overload
        def __call__(self, value: Int128) -> Int128:...
        # Method ReverseEndianness(value : Byte) was skipped since it collides with above method
        # Method ReverseEndianness(value : UInt16) was skipped since it collides with above method
        # Method ReverseEndianness(value : UInt32) was skipped since it collides with above method
        # Method ReverseEndianness(value : UInt64) was skipped since it collides with above method
        @typing.overload
        def __call__(self, value: UInt128) -> UInt128:...
        # Method ReverseEndianness(value : IntPtr) was skipped since it collides with above method
        @typing.overload
        def __call__(self, value: UIntPtr) -> UIntPtr:...
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[int], destination: Span_1[int]) -> None:...
        # Method ReverseEndianness(source : ReadOnlySpan`1, destination : Span`1) was skipped since it collides with above method
        # Method ReverseEndianness(source : ReadOnlySpan`1, destination : Span`1) was skipped since it collides with above method
        # Method ReverseEndianness(source : ReadOnlySpan`1, destination : Span`1) was skipped since it collides with above method
        # Method ReverseEndianness(source : ReadOnlySpan`1, destination : Span`1) was skipped since it collides with above method
        # Method ReverseEndianness(source : ReadOnlySpan`1, destination : Span`1) was skipped since it collides with above method
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[UInt128], destination: Span_1[UInt128]) -> None:...
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[Int128], destination: Span_1[Int128]) -> None:...
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[UIntPtr], destination: Span_1[UIntPtr]) -> None:...
        # Method ReverseEndianness(source : ReadOnlySpan`1, destination : Span`1) was skipped since it collides with above method





import abc
from System.Collections.Generic import List_1
from System.IO import FileInfo
from SIMULTAN.Projects import ProjectData

class PPathIO(abc.ABC):
    @staticmethod
    def Read(file: FileInfo) -> List_1[str]: ...
    @staticmethod
    def Write(file: FileInfo, projectData: ProjectData) -> None: ...




import typing

class AssemblyHashAlgorithm(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    None_ : AssemblyHashAlgorithm # 0
    MD5 : AssemblyHashAlgorithm # 32771
    SHA1 : AssemblyHashAlgorithm # 32772
    SHA256 : AssemblyHashAlgorithm # 32780
    SHA384 : AssemblyHashAlgorithm # 32781
    SHA512 : AssemblyHashAlgorithm # 32782


class AssemblyVersionCompatibility(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    SameMachine : AssemblyVersionCompatibility # 1
    SameProcess : AssemblyVersionCompatibility # 2
    SameDomain : AssemblyVersionCompatibility # 3




import inspect
import math
import os
from time import sleep

import PySimultan2.geometry
from PySimultan2 import PythonMapper, DataModel
from PySimultan2.taxonomy_maps import Content, TaxonomyMap
from PySimultan2.default_types import ComponentList

from PySimultan2.geometry.geometry_base import (GeometryModel)
from PySimultan2.geometry.utils import create_cube

from SIMULTAN import *
from SIMULTAN.Data import *
from SIMULTAN.Data.Geometry import *

project_dir = os.environ.get('PROJECT_DIR', '')
project_name = os.environ.get('PROJECT_NAME', 'ida_ice_structure.simultan')


def create_geometry_model(name='new_geometry_test', data_model=None):
    return GeometryModel(name=name,
                         data_model=data_model)



#####################################################################
# Create classes
#####################################################################

def create_contents():

    contents = {}

    contents['temperature_boundary_condition'] = Content(text_or_key='temperature_boundary_condition',
                                                         property_name='temperature_boundary_condition',
                                                         type=None,
                                                         unit='',
                                                         documentation='',
                                                         component_policy='reference')

    contents['construction'] = Content(text_or_key='Constructional_Element',
                                       property_name='construction',
                                       type=None,
                                       unit='',
                                       documentation='',
                                       component_policy='reference')

    contents['material'] = Content(text_or_key='material',
                                   property_name='material',
                                   type=None,
                                   unit='',
                                   documentation='',
                                   component_policy='reference')

    contents['thickness'] = Content(text_or_key='thickness',
                                    property_name='thickness',
                                    type=None,
                                    unit='',
                                    documentation='',
                                    component_policy='subcomponent',
                                    taxonomy_key='Building',
                                    taxonomy_name='Building')

    contents['thermal_conductivity'] = Content(text_or_key='thermal_conductivity',
                                               property_name='thermal_conductivity',
                                               type=float,
                                               unit='W/mK',
                                               documentation='',
                                               component_policy='subcomponent',
                                               taxonomy_name='Building',
                                               taxonomy_key='Building',
                                               )

    contents['specific_heat'] = Content(text_or_key='specific_heat',
                                        property_name='specific_heat',
                                        type=float,
                                        unit='J/kgK',
                                        documentation='',
                                        component_policy='subcomponent',
                                        taxonomy_name='Building',
                                        taxonomy_key='Building',
                                        )

    contents['density'] = Content(text_or_key='density',
                                  property_name='density',
                                  type=float,
                                  unit='kg/m3',
                                  documentation='',
                                  component_policy='subcomponent',
                                  taxonomy_name='Building',
                                  taxonomy_key='Building',
                                  )

    contents['alphaS'] = Content(text_or_key='alphaS',
                                 property_name='alphaS',
                                 type=float,
                                 unit='-',
                                 documentation='sound absorption coefficent',
                                 component_policy='subcomponent',
                                 ValueMin=0.0,
                                 ValueMax=1.0,
                                 taxonomy_name='Building',
                                 taxonomy_key='Building',
                                 )

    contents['alpha0'] = Content(text_or_key='alpha0',
                                 property_name='alpha0',
                                 type=float,
                                 unit='-',
                                 documentation='sound absorption coefficent',
                                 component_policy='subcomponent',
                                 ValueMin=0.0,
                                 ValueMax=1.0)

    contents['alphaN'] = Content(text_or_key='alphaN',
                                 property_name='alphaN',
                                 type=float,
                                 unit='-',
                                 documentation='sound absorption coefficent',
                                 component_policy='subcomponent',
                                 ValueMin=0.0,
                                 ValueMax=1.0)

    contents['r_se_cl'] = Content(text_or_key='r_se_cl',
                                  property_name='r_se_cl',
                                  type=float,
                                  unit='m2K/W',
                                  documentation='',
                                  component_policy='subcomponent')

    contents['r_si_cl'] = Content(text_or_key='r_si_cl',
                                  property_name='r_si_cl',
                                  type=float,
                                  unit='m2K/W',
                                  documentation='',
                                  component_policy='subcomponent')

    contents['temperature_cl'] = Content(text_or_key='temperature_cl',
                                         property_name='temperature_cl',
                                         type=None,
                                         unit='',
                                         documentation='',
                                         component_policy='reference')

    contents['RT'] = Content(text_or_key='RT_cl',
                             property_name='RT',
                             type=float,
                             unit='s',
                             documentation='reverberation time in s',
                             component_policy='subcomponent')


    contents['Aopt'] = Content(text_or_key='Aopt_cl',
                               property_name='Aopt',
                               type=float,
                               unit='m²',
                               documentation='equivalent absorption area in m²',
                               component_policy='subcomponent')

    contents['RTopt'] = Content(text_or_key='RTopt_cl',
                               property_name='RTopt',
                               type=float,
                               unit='s',
                               documentation='optimal reverberation time in s',
                               component_policy='subcomponent')

    return contents


def create_classes_and_map():

    mapper = PythonMapper(module='ida_ice_structure')


    class Construction_Assignement_List(ComponentList):
        pass


    class ConstructionAssignment:

        def __init__(self,
                     name,
                     temperature_boundary_condition,
                     construction,
                     *args,
                     **kwargs
                     ):
            self.name = name
            self.temperature_boundary_condition = temperature_boundary_condition
            self.construction = construction


    class Construction(ComponentList):
        """
        Class representing a construction as a list of layers
        """
        def __init__(self,
                     name,
                     r_se_cl: float = None,
                     r_si_cl: float = None,
                     alpha0: float = None,
                     alphaN: float = None,
                     *args,
                     **kwargs
                     ):
            super().__init__(*args, **kwargs)
            self.name = name
            self.r_se_cl = r_se_cl
            self.r_si_cl = r_si_cl
            self.alpha0 = alpha0
            self.alphaN = alphaN


    class Layer:

        def __init__(self,
                     name,
                     material,
                     thickness,
                     *args,
                     **kwargs
                     ):
            self.name = name
            self.material = material
            self.thickness = thickness


    class Material:

        def __init__(self,
                     name,
                     density: float,
                     specific_heat: float,
                     thermal_conductivity: float,
                     alphaS: float = None,
                     *args,
                     **kwargs
                     ):
            self.name = name
            self.density = density
            self.specific_heat = specific_heat
            self.thermal_conductivity = thermal_conductivity
            self.alphaS = alphaS


    class TemperatureBoundaryCondition:

        def __init__(self,
                    name: str,
                    temperature_cl: float,
                    *args,
                    **kwargs
                    ):
            self.name = name
            self.temperature_cl = temperature_cl


    class Room_Cl:

        def __init__(self,
                    name: str,
                    RT: float = None,
                    RTopt: float = None,
                    Aopt: float = None,
                    *args,
                    **kwargs
                    ):
            self.name = name
            self.RT = RT
            self.RTopt = RTopt
            self.Aopt = Aopt

        def calculate_RT(self):
            try:
                volume=self.associated_geometry[0].volume
                self.RTopt=0.5+0.05*math.log(volume)  # EN 12354
                self.Aopt=0.163*volume/self.RTopt

                print(self.name)
                EquivArea=0
                anzfaces=len(self.associated_geometry[0].faces)
                for face in self.associated_geometry[0].faces:
                    alphaS=0.0
                    if len(face.components)>0 :
                        construction=face.components[0].construction
                        orient1=face.orientation

                        PVolumeTest=self.associated_geometry[0]._wrapped_object
                        PFace = next((PF for PF in  PVolumeTest.Faces if PF.Face == face._wrapped_object), None)
                        orient2=PFace.Orientation

                        print(construction.name," Orientierung", orient1,orient2)


                        if orient2 == orient2.Forward:
                            if construction.alpha0 is not None:

                                if construction.alpha0>0 and construction.alpha0<1:
                                    alphaS=construction.alpha0
                                else:
                                    construction.alpha0=99.0
                                    construction.data[0].material.alphaS=99.0
                                    alphaS=-1
                            else:
                                construction.alpha0=99.0
                                alphaS=-1


                            print("Erste Schichte",alphaS)
                        else:
                            if construction.alphaN is not None:
                                if construction.alphaN>0 and construction.alphaN<1:
                                    alphaS = construction.alphaN
                                else:
                                    construction.alphaN=99.0
                                    construction.data[len(construction.data)-1].material.alphaS=99.0

                                    alphaS=-1

                            else:
                                construction.alphaN=99.0
                                construction.data[len(construction.data)-1].material.alphaS=99.0
                                alphaS=-1

                            print("Letzte Schichte",alphaS)
                        for layer in construction.data:
                            if layer is not None:
                                print(layer.name)
                            else:
                                print("ERROR Layer ist None")

                    if alphaS is not None: EquivArea += face.area*alphaS



                if EquivArea>0:
                    RT= 0.163*volume / EquivArea
                else:
                    RT=-1.0

                self.RT = RT
            except Exception as e:
                raise e

            return RT



    contents = create_contents()

    construction_assignment_list_map = TaxonomyMap(taxonomy_name='Building',
                                                   taxonomy_key='Building',
                                                   taxonomy_entry_name='Constructional Assignment List',
                                                   taxonomy_entry_key='Constructional_Assignment_List',
                                                   content=[]
                                                   )

    mapper.register(taxonomy=construction_assignment_list_map.taxonomy_entry_key,
                    cls=Construction_Assignement_List,
                    taxonomy_map=construction_assignment_list_map)


    construction_map = TaxonomyMap(taxonomy_name='Building',
                                   taxonomy_key='Building',
                                   taxonomy_entry_name='Constructional Element',
                                   taxonomy_entry_key='Constructional_Element',
                                   content=[contents['r_se_cl'],
                                            contents['r_si_cl'],
                                            contents['alpha0'],
                                            contents['alphaN'],
                                            ]
                                   )


    mapper.register(taxonomy=construction_map.taxonomy_entry_key,
                    cls=Construction,
                    taxonomy_map=construction_map)

    construction_assignment_map = TaxonomyMap(taxonomy_name='Building',
                                              taxonomy_key='Building',
                                              taxonomy_entry_name='Constructional Assignment',
                                              taxonomy_entry_key='Constructional_Assignment',
                                              content=[contents['temperature_boundary_condition'],
                                                       contents['construction']
                                                       ]
                                              )

    mapper.register(taxonomy=construction_assignment_map.taxonomy_entry_key,
                    cls=ConstructionAssignment,
                    taxonomy_map=construction_assignment_map)

    layer_map = TaxonomyMap(taxonomy_name='Reserved Slots',
                            taxonomy_key='resslot',
                            taxonomy_entry_name='Layer',
                            taxonomy_entry_key='layer',
                            content=[contents['material'],
                                     contents['thickness']
                                     ]
                            )

    mapper.register(taxonomy=layer_map.taxonomy_entry_key,
                    cls=Layer,
                    taxonomy_map=layer_map)


    material_map = TaxonomyMap(taxonomy_name='Reserved Slots',
                                 taxonomy_key='resslot',
                                 taxonomy_entry_name='Material',
                                 taxonomy_entry_key='material',
                                 content=[contents['density'],
                                         contents['specific_heat'],
                                         contents['thermal_conductivity'],
                                         contents['alphaS'],
                                         ]
                                 )

    mapper.register(taxonomy=material_map.taxonomy_entry_key,
                    cls=Material,
                    taxonomy_map=material_map)

    temperature_boundary_condition_map = TaxonomyMap(taxonomy_name='PySimultan_RoomAku',
                                                     taxonomy_key='PySimultan_RoomAku',
                                                     taxonomy_entry_name='TemperatureBoundaryCondition_name',
                                                     taxonomy_entry_key='TemperatureBoundaryCondition_key',
                                                     content=[
                                                         contents['temperature_cl']
                                                     ]
                                                     )

    mapper.register(taxonomy=temperature_boundary_condition_map.taxonomy_entry_key,
                    cls=TemperatureBoundaryCondition,
                    taxonomy_map=temperature_boundary_condition_map)

    room_map = TaxonomyMap(taxonomy_name='PySimultan_RoomAku',
                           taxonomy_key='PySimultan_RoomAku',
                           taxonomy_entry_name='Room_tax_name',
                           taxonomy_entry_key='Room_tax_key',
                           content=[
                                    contents['RT'],
                                    contents['RTopt'],
                                    contents['Aopt']
                                   ]
                           )

    mapper.register(taxonomy=room_map.taxonomy_entry_key,
                    cls=Room_Cl,
                    taxonomy_map=room_map)

    return mapper




def create_model(project_name=None,mapper=None):

    if project_name is None:
        project_name = project_path

    data_model = DataModel.create_new_project(project_path=os.path.join(project_dir, project_name),
                                              user_name='admin',
                                              password='admin')

    MappedConstructionAssignmentList = mapper.get_mapped_class('Constructional_Assignment_List')
    MappedConstructionAssignment = mapper.get_mapped_class('Constructional_Assignment')
    MappedConstruction = mapper.get_mapped_class('Constructional_Element')
    MappedLayer = mapper.get_mapped_class('layer')
    MappedMaterial = mapper.get_mapped_class('material')
    MappedTemperatureBoundaryCondition = mapper.get_mapped_class('TemperatureBoundaryCondition_key')
    MappedRoom = mapper.get_mapped_class('Room_tax_key')

    concrete = MappedMaterial(name='concrete',
                              density=2600,
                              specific_heat=800,
                              thermal_conductivity=1.5)

    insulation = MappedMaterial(name='insulation',
                                density=150,
                                specific_heat=1500,
                                thermal_conductivity=0.03)

    plaster = MappedMaterial(name='plaster',
                             density=1200,
                             specific_heat=700,
                             thermal_conductivity=0.60)

    t_out_1 = MappedTemperatureBoundaryCondition(name='t_out_1',
                                                 temperature_cl=273.15)

    t_out_2 = MappedTemperatureBoundaryCondition(name='t_out_2',
                                                 temperature_cl=283.15)

    # wall1
    wall1 = MappedConstruction(name='wall1',
                               r_se_cl=0.04,
                               r_si_cl=0.13)
    wall1_layers = [
        MappedLayer(name='concrete_layer',
                    material=concrete,
                    thickness=0.2),
        MappedLayer(name='insulation_layer',
                    material=insulation,
                    thickness=0.1),
        MappedLayer(name='plaster_layer',
                    material=plaster,
                    thickness=0.04)
    ]
    wall1.extend(wall1_layers)

    wall2_layers = [
        MappedLayer(name='concrete_layer',
                    material=concrete,
                    thickness=0.15),
        MappedLayer(name='insulation_layer',
                    material=insulation,
                    thickness=0.2),
    ]

    wall2 = MappedConstruction(name='wall2',
                               r_se_cl=0.04,
                               r_si_cl=0.13)
    wall2.extend(wall2_layers)

    bauteilzuweisung = MappedConstructionAssignmentList(name='Bauteilzuweisung')

    con1_zuw = MappedConstructionAssignment(name='con1_zuw',
                                            temperature_boundary_condition=t_out_1,
                                            construction=wall1)

    con2_zuw = MappedConstructionAssignment(name='con2_zuw',
                                            temperature_boundary_condition=t_out_2,
                                            construction=wall1)

    bauteilzuweisung.extend([con1_zuw, con2_zuw])

    Room1 = MappedRoom(name='Room1',
                       RT=3.0)

    geo_model = create_geometry_model(name='new_geometry_test',
                                      data_model=data_model)

    cube = create_cube(data_model, geo_model, scale=10)

    Room1.associate(cube)
    Room1.calculate_RT()

    con1_zuw.associate(cube.faces[0])
    con2_zuw.associate(cube.faces[1])

    data_model.save()
    data_model.cleanup()



def test_loaded_model(project_name=None,mapper=None):

    if project_name is None:
        project_name = project_path

    data_model = DataModel(project_path=project_name,
                           user_name='admin',
                           password='admin')

    mapper.load_undefined = False
    #mapper.load_undefined = True

    typed_data = data_model.get_typed_data(mapper=mapper,
                                           create_all=True)



    zone_cls = mapper.get_mapped_class('Room_tax_key')

    for zone in zone_cls.cls_instances:
        print("************************************************")
        print(zone.name)

        RT = zone.calculate_RT()
        volume = zone.associated_geometry[0].volume
        print(zone.name,"  RT ",RT,"Volumen",volume)


    data_model.save()

    data_model.cleanup()
    mapper.clear()
    del data_model


if __name__ == '__main__':

    RoomAcu_mapper = create_classes_and_map()

    create_model(project_name="TestNEU3.simultan", mapper=RoomAcu_mapper)

    # sleep(1)
    # test_loaded_model(project_name="241217_DigiTwin_as.simultan", mapper=RoomAcu_mapper)

    # test_loaded_model(project_name="Testmodell.simultan", mapper=RoomAcu_mapper)
    # sleep(1)



import typing, clr, abc
from System.Collections.Generic import IReadOnlyList_1, IList_1, IEnumerator_1, IEnumerable_1, List_1, IReadOnlyDictionary_2, IDictionary_2, KeyValuePair_2, IReadOnlyCollection_1, ICollection_1
from System.Collections import IList, IDictionary, ICollection
from System import Array_1
from System.ComponentModel import INotifyPropertyChanged
from System.Collections.Specialized import INotifyCollectionChanged

class Collection_GenericClasses(abc.ABCMeta):
    Generic_Collection_GenericClasses_Collection_1_T = typing.TypeVar('Generic_Collection_GenericClasses_Collection_1_T')
    def __getitem__(self, types : typing.Type[Generic_Collection_GenericClasses_Collection_1_T]) -> typing.Type[Collection_1[Generic_Collection_GenericClasses_Collection_1_T]]: ...

Collection : Collection_GenericClasses

Collection_1_T = typing.TypeVar('Collection_1_T')
class Collection_1(typing.Generic[Collection_1_T], IReadOnlyList_1[Collection_1_T], IList_1[Collection_1_T], IList):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, list: IList_1[Collection_1_T]) -> None: ...
    @property
    def Count(self) -> int: ...
    @property
    def Item(self) -> Collection_1_T: ...
    @Item.setter
    def Item(self, value: Collection_1_T) -> Collection_1_T: ...
    def Add(self, item: Collection_1_T) -> None: ...
    def Clear(self) -> None: ...
    def Contains(self, item: Collection_1_T) -> bool: ...
    def CopyTo(self, array: Array_1[Collection_1_T], index: int) -> None: ...
    def GetEnumerator(self) -> IEnumerator_1[Collection_1_T]: ...
    def IndexOf(self, item: Collection_1_T) -> int: ...
    def Insert(self, index: int, item: Collection_1_T) -> None: ...
    def Remove(self, item: Collection_1_T) -> bool: ...
    def RemoveAt(self, index: int) -> None: ...


class ObservableCollection_GenericClasses(abc.ABCMeta):
    Generic_ObservableCollection_GenericClasses_ObservableCollection_1_T = typing.TypeVar('Generic_ObservableCollection_GenericClasses_ObservableCollection_1_T')
    def __getitem__(self, types : typing.Type[Generic_ObservableCollection_GenericClasses_ObservableCollection_1_T]) -> typing.Type[ObservableCollection_1[Generic_ObservableCollection_GenericClasses_ObservableCollection_1_T]]: ...

ObservableCollection : ObservableCollection_GenericClasses

ObservableCollection_1_T = typing.TypeVar('ObservableCollection_1_T')
class ObservableCollection_1(typing.Generic[ObservableCollection_1_T], Collection_1[ObservableCollection_1_T], INotifyPropertyChanged, INotifyCollectionChanged):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, collection: IEnumerable_1[ObservableCollection_1_T]) -> None: ...
    @typing.overload
    def __init__(self, list: List_1[ObservableCollection_1_T]) -> None: ...
    @property
    def Count(self) -> int: ...
    @property
    def Item(self) -> ObservableCollection_1_T: ...
    @Item.setter
    def Item(self, value: ObservableCollection_1_T) -> ObservableCollection_1_T: ...
    def Move(self, oldIndex: int, newIndex: int) -> None: ...


class ReadOnlyCollection_GenericClasses(abc.ABCMeta):
    Generic_ReadOnlyCollection_GenericClasses_ReadOnlyCollection_1_T = typing.TypeVar('Generic_ReadOnlyCollection_GenericClasses_ReadOnlyCollection_1_T')
    def __getitem__(self, types : typing.Type[Generic_ReadOnlyCollection_GenericClasses_ReadOnlyCollection_1_T]) -> typing.Type[ReadOnlyCollection_1[Generic_ReadOnlyCollection_GenericClasses_ReadOnlyCollection_1_T]]: ...

ReadOnlyCollection : ReadOnlyCollection_GenericClasses

ReadOnlyCollection_1_T = typing.TypeVar('ReadOnlyCollection_1_T')
class ReadOnlyCollection_1(typing.Generic[ReadOnlyCollection_1_T], IReadOnlyList_1[ReadOnlyCollection_1_T], IList_1[ReadOnlyCollection_1_T], IList):
    def __init__(self, list: IList_1[ReadOnlyCollection_1_T]) -> None: ...
    @property
    def Count(self) -> int: ...
    @classmethod
    @property
    def Empty(cls) -> ReadOnlyCollection_1[ReadOnlyCollection_1_T]: ...
    @property
    def Item(self) -> ReadOnlyCollection_1_T: ...
    def Contains(self, value: ReadOnlyCollection_1_T) -> bool: ...
    def CopyTo(self, array: Array_1[ReadOnlyCollection_1_T], index: int) -> None: ...
    def GetEnumerator(self) -> IEnumerator_1[ReadOnlyCollection_1_T]: ...
    def IndexOf(self, value: ReadOnlyCollection_1_T) -> int: ...


class ReadOnlyDictionary_GenericClasses(abc.ABCMeta):
    Generic_ReadOnlyDictionary_GenericClasses_ReadOnlyDictionary_2_TKey = typing.TypeVar('Generic_ReadOnlyDictionary_GenericClasses_ReadOnlyDictionary_2_TKey')
    Generic_ReadOnlyDictionary_GenericClasses_ReadOnlyDictionary_2_TValue = typing.TypeVar('Generic_ReadOnlyDictionary_GenericClasses_ReadOnlyDictionary_2_TValue')
    def __getitem__(self, types : typing.Tuple[typing.Type[Generic_ReadOnlyDictionary_GenericClasses_ReadOnlyDictionary_2_TKey], typing.Type[Generic_ReadOnlyDictionary_GenericClasses_ReadOnlyDictionary_2_TValue]]) -> typing.Type[ReadOnlyDictionary_2[Generic_ReadOnlyDictionary_GenericClasses_ReadOnlyDictionary_2_TKey, Generic_ReadOnlyDictionary_GenericClasses_ReadOnlyDictionary_2_TValue]]: ...

ReadOnlyDictionary : ReadOnlyDictionary_GenericClasses

ReadOnlyDictionary_2_TKey = typing.TypeVar('ReadOnlyDictionary_2_TKey')
ReadOnlyDictionary_2_TValue = typing.TypeVar('ReadOnlyDictionary_2_TValue')
class ReadOnlyDictionary_2(typing.Generic[ReadOnlyDictionary_2_TKey, ReadOnlyDictionary_2_TValue], IReadOnlyDictionary_2[ReadOnlyDictionary_2_TKey, ReadOnlyDictionary_2_TValue], IDictionary_2[ReadOnlyDictionary_2_TKey, ReadOnlyDictionary_2_TValue], IDictionary):
    def __init__(self, dictionary: IDictionary_2[ReadOnlyDictionary_2_TKey, ReadOnlyDictionary_2_TValue]) -> None: ...
    @property
    def Count(self) -> int: ...
    @classmethod
    @property
    def Empty(cls) -> ReadOnlyDictionary_2[ReadOnlyDictionary_2_TKey, ReadOnlyDictionary_2_TValue]: ...
    @property
    def Item(self) -> ReadOnlyDictionary_2_TValue: ...
    @property
    def Keys(self) -> ReadOnlyDictionary_2.KeyCollection_2[ReadOnlyDictionary_2_TKey, ReadOnlyDictionary_2_TValue]: ...
    @property
    def Values(self) -> ReadOnlyDictionary_2.ValueCollection_2[ReadOnlyDictionary_2_TKey, ReadOnlyDictionary_2_TValue]: ...
    def ContainsKey(self, key: ReadOnlyDictionary_2_TKey) -> bool: ...
    def GetEnumerator(self) -> IEnumerator_1[KeyValuePair_2[ReadOnlyDictionary_2_TKey, ReadOnlyDictionary_2_TValue]]: ...
    def TryGetValue(self, key: ReadOnlyDictionary_2_TKey, value: clr.Reference[ReadOnlyDictionary_2_TValue]) -> bool: ...

    KeyCollection_GenericClasses_ReadOnlyDictionary_2_TKey = typing.TypeVar('KeyCollection_GenericClasses_ReadOnlyDictionary_2_TKey')
    KeyCollection_GenericClasses_ReadOnlyDictionary_2_TValue = typing.TypeVar('KeyCollection_GenericClasses_ReadOnlyDictionary_2_TValue')
    class KeyCollection_GenericClasses(typing.Generic[KeyCollection_GenericClasses_ReadOnlyDictionary_2_TKey, KeyCollection_GenericClasses_ReadOnlyDictionary_2_TValue], abc.ABCMeta):
        KeyCollection_GenericClasses_ReadOnlyDictionary_2_TKey = ReadOnlyDictionary_2.KeyCollection_GenericClasses_ReadOnlyDictionary_2_TKey
        KeyCollection_GenericClasses_ReadOnlyDictionary_2_TValue = ReadOnlyDictionary_2.KeyCollection_GenericClasses_ReadOnlyDictionary_2_TValue
        def __call__(self) -> ReadOnlyDictionary_2.KeyCollection_2[KeyCollection_GenericClasses_ReadOnlyDictionary_2_TKey, KeyCollection_GenericClasses_ReadOnlyDictionary_2_TValue]: ...

    KeyCollection : KeyCollection_GenericClasses[ReadOnlyDictionary_2_TKey, ReadOnlyDictionary_2_TValue]

    KeyCollection_2_TKey = typing.TypeVar('KeyCollection_2_TKey')
    KeyCollection_2_TValue = typing.TypeVar('KeyCollection_2_TValue')
    class KeyCollection_2(typing.Generic[KeyCollection_2_TKey, KeyCollection_2_TValue], IReadOnlyCollection_1[KeyCollection_2_TKey], ICollection_1[KeyCollection_2_TKey], ICollection):
        KeyCollection_2_TKey = ReadOnlyDictionary_2.KeyCollection_2_TKey
        KeyCollection_2_TValue = ReadOnlyDictionary_2.KeyCollection_2_TValue
        @property
        def Count(self) -> int: ...
        def Contains(self, item: KeyCollection_2_TKey) -> bool: ...
        def CopyTo(self, array: Array_1[KeyCollection_2_TKey], arrayIndex: int) -> None: ...
        def GetEnumerator(self) -> IEnumerator_1[KeyCollection_2_TKey]: ...


    ValueCollection_GenericClasses_ReadOnlyDictionary_2_TKey = typing.TypeVar('ValueCollection_GenericClasses_ReadOnlyDictionary_2_TKey')
    ValueCollection_GenericClasses_ReadOnlyDictionary_2_TValue = typing.TypeVar('ValueCollection_GenericClasses_ReadOnlyDictionary_2_TValue')
    class ValueCollection_GenericClasses(typing.Generic[ValueCollection_GenericClasses_ReadOnlyDictionary_2_TKey, ValueCollection_GenericClasses_ReadOnlyDictionary_2_TValue], abc.ABCMeta):
        ValueCollection_GenericClasses_ReadOnlyDictionary_2_TKey = ReadOnlyDictionary_2.ValueCollection_GenericClasses_ReadOnlyDictionary_2_TKey
        ValueCollection_GenericClasses_ReadOnlyDictionary_2_TValue = ReadOnlyDictionary_2.ValueCollection_GenericClasses_ReadOnlyDictionary_2_TValue
        def __call__(self) -> ReadOnlyDictionary_2.ValueCollection_2[ValueCollection_GenericClasses_ReadOnlyDictionary_2_TKey, ValueCollection_GenericClasses_ReadOnlyDictionary_2_TValue]: ...

    ValueCollection : ValueCollection_GenericClasses[ReadOnlyDictionary_2_TKey, ReadOnlyDictionary_2_TValue]

    ValueCollection_2_TKey = typing.TypeVar('ValueCollection_2_TKey')
    ValueCollection_2_TValue = typing.TypeVar('ValueCollection_2_TValue')
    class ValueCollection_2(typing.Generic[ValueCollection_2_TKey, ValueCollection_2_TValue], IReadOnlyCollection_1[ValueCollection_2_TValue], ICollection_1[ValueCollection_2_TValue], ICollection):
        ValueCollection_2_TKey = ReadOnlyDictionary_2.ValueCollection_2_TKey
        ValueCollection_2_TValue = ReadOnlyDictionary_2.ValueCollection_2_TValue
        @property
        def Count(self) -> int: ...
        def CopyTo(self, array: Array_1[ValueCollection_2_TValue], arrayIndex: int) -> None: ...
        def GetEnumerator(self) -> IEnumerator_1[ValueCollection_2_TValue]: ...



class ReadOnlyObservableCollection_GenericClasses(abc.ABCMeta):
    Generic_ReadOnlyObservableCollection_GenericClasses_ReadOnlyObservableCollection_1_T = typing.TypeVar('Generic_ReadOnlyObservableCollection_GenericClasses_ReadOnlyObservableCollection_1_T')
    def __getitem__(self, types : typing.Type[Generic_ReadOnlyObservableCollection_GenericClasses_ReadOnlyObservableCollection_1_T]) -> typing.Type[ReadOnlyObservableCollection_1[Generic_ReadOnlyObservableCollection_GenericClasses_ReadOnlyObservableCollection_1_T]]: ...

ReadOnlyObservableCollection : ReadOnlyObservableCollection_GenericClasses

ReadOnlyObservableCollection_1_T = typing.TypeVar('ReadOnlyObservableCollection_1_T')
class ReadOnlyObservableCollection_1(typing.Generic[ReadOnlyObservableCollection_1_T], ReadOnlyCollection_1[ReadOnlyObservableCollection_1_T], INotifyPropertyChanged, INotifyCollectionChanged):
    def __init__(self, list: ObservableCollection_1[ReadOnlyObservableCollection_1_T]) -> None: ...
    @property
    def Count(self) -> int: ...
    @classmethod
    @property
    def Empty(cls) -> ReadOnlyObservableCollection_1[ReadOnlyObservableCollection_1_T]: ...
    @property
    def Item(self) -> ReadOnlyObservableCollection_1_T: ...




import typing
from SIMULTAN.Data.Taxonomy import SimTaxonomyEntry
from SIMULTAN.Projects import ProjectData
from SIMULTAN.Data.Geometry import GeometryModel, OffsetAlgorithm
from SIMULTAN.Data.Assets import ResourceFileEntry
from System.Collections.Generic import List_1, IEnumerable_1, Dictionary_2
from System import Array_1

class SimGeoIO:
    def __init__(self) -> None: ...
    @classmethod
    @property
    def SimGeoVersion(cls) -> int: ...
    @staticmethod
    def GetLegacyParentTaxonomyEntry(projectData: ProjectData) -> SimTaxonomyEntry: ...
    @staticmethod
    def Load(geometryFile: ResourceFileEntry, projectData: ProjectData, errors: List_1[SimGeoIOError], offsetAlg: OffsetAlgorithm = ...) -> GeometryModel: ...
    @staticmethod
    def MigrateAfterImport(geometryFiles: IEnumerable_1[ResourceFileEntry], projectData: ProjectData, errors: List_1[SimGeoIOError], importedKeysLookup: Dictionary_2[int, int], importedFilesLookup: Dictionary_2[str, str], offsetAlg: OffsetAlgorithm = ...) -> None: ...
    @staticmethod
    def Save(model: GeometryModel, file: ResourceFileEntry, mode: SimGeoIO.WriteMode) -> bool: ...

    class WriteMode(typing.SupportsInt):
        @typing.overload
        def __init__(self, value : int) -> None: ...
        @typing.overload
        def __init__(self, value : int, force_if_true: bool) -> None: ...
        def __int__(self) -> int: ...
        
        # Values:
        Plaintext : SimGeoIO.WriteMode # 0



class SimGeoIOError:
    def __init__(self, reason: SimGeoIOErrorReason, data: Array_1[typing.Any]) -> None: ...
    @property
    def Data(self) -> Array_1[typing.Any]: ...
    @property
    def Reason(self) -> SimGeoIOErrorReason: ...


class SimGeoIOErrorReason(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    InvalidLinkedModel : SimGeoIOErrorReason # 0
    ReferenceConvertFailed : SimGeoIOErrorReason # 1
    ModelWithSameId : SimGeoIOErrorReason # 2




import typing, clr, abc
from System.Buffers import OperationStatus
from System import ReadOnlySpan_1, Span_1, IFormatProvider

class Utf8(abc.ABC):
    @staticmethod
    def FromUtf16(source: ReadOnlySpan_1[str], destination: Span_1[int], charsRead: clr.Reference[int], bytesWritten: clr.Reference[int], replaceInvalidSequences: bool = ..., isFinalBlock: bool = ...) -> OperationStatus: ...
    @staticmethod
    def IsValid(value: ReadOnlySpan_1[int]) -> bool: ...
    @staticmethod
    def ToUtf16(source: ReadOnlySpan_1[int], destination: Span_1[str], bytesRead: clr.Reference[int], charsWritten: clr.Reference[int], replaceInvalidSequences: bool = ..., isFinalBlock: bool = ...) -> OperationStatus: ...
    # Skipped TryWrite due to it being static, abstract and generic.

    TryWrite : TryWrite_MethodGroup
    class TryWrite_MethodGroup:
        @typing.overload
        def __call__(self, destination: Span_1[int], handler: clr.Reference[Utf8.TryWriteInterpolatedStringHandler], bytesWritten: clr.Reference[int]) -> bool:...
        @typing.overload
        def __call__(self, destination: Span_1[int], provider: IFormatProvider, handler: clr.Reference[Utf8.TryWriteInterpolatedStringHandler], bytesWritten: clr.Reference[int]) -> bool:...


    class TryWriteInterpolatedStringHandler:
        @typing.overload
        def __init__(self, literalLength: int, formattedCount: int, destination: Span_1[int], provider: IFormatProvider, shouldAppend: clr.Reference[bool]) -> None: ...
        @typing.overload
        def __init__(self, literalLength: int, formattedCount: int, destination: Span_1[int], shouldAppend: clr.Reference[bool]) -> None: ...
        def AppendLiteral(self, value: str) -> bool: ...
        # Skipped AppendFormatted due to it being static, abstract and generic.

        AppendFormatted : AppendFormatted_MethodGroup
        class AppendFormatted_MethodGroup:
            def __getitem__(self, t:typing.Type[AppendFormatted_1_T1]) -> AppendFormatted_1[AppendFormatted_1_T1]: ...

            AppendFormatted_1_T1 = typing.TypeVar('AppendFormatted_1_T1')
            class AppendFormatted_1(typing.Generic[AppendFormatted_1_T1]):
                AppendFormatted_1_T = Utf8.TryWriteInterpolatedStringHandler.AppendFormatted_MethodGroup.AppendFormatted_1_T1
                @typing.overload
                def __call__(self, value: AppendFormatted_1_T) -> bool:...
                @typing.overload
                def __call__(self, value: AppendFormatted_1_T, alignment: int) -> bool:...
                @typing.overload
                def __call__(self, value: AppendFormatted_1_T, format: str) -> bool:...
                @typing.overload
                def __call__(self, value: AppendFormatted_1_T, alignment: int, format: str) -> bool:...

            @typing.overload
            def __call__(self, utf8Value: ReadOnlySpan_1[int]) -> bool:...
            @typing.overload
            def __call__(self, value: ReadOnlySpan_1[str]) -> bool:...
            @typing.overload
            def __call__(self, value: str) -> bool:...
            @typing.overload
            def __call__(self, utf8Value: ReadOnlySpan_1[int], alignment: int = ..., format: str = ...) -> bool:...
            @typing.overload
            def __call__(self, value: ReadOnlySpan_1[str], alignment: int = ..., format: str = ...) -> bool:...
            @typing.overload
            def __call__(self, value: str, alignment: int = ..., format: str = ...) -> bool:...
            @typing.overload
            def __call__(self, value: typing.Any, alignment: int = ..., format: str = ...) -> bool:...






from PySimultan2.py_simultan_ui.main_ui import run_ui
from PySimultan2.py_simultan_ui.core import mapper
from PySimultan2.taxonomy_maps import TaxonomyMap, Content
from PySimultan2.py_simultan_ui.core.method_mapper import method_mapper


class TestClass(object):

    def __init__(self, *args, **kwargs):
        self.param_1 = kwargs.get('param_1', 1)
        self.param_2 = kwargs.get('param_2', 2)
        self.c = kwargs.get('c', None)

    def test_method(self):
        self.c = self.param_1 + self.param_2


c_param_1 = Content(text_or_key='param_1',
                    property_name='param_1',
                    type=None,
                    unit='',
                    documentation='param_1')

c_param_2 = Content(text_or_key='param_2',
                    property_name='param_2',
                    type=None,
                    unit='',
                    documentation='param_2')
#
c_c = Content(text_or_key='c',
              property_name='c',
              type=None,
              unit='',
              documentation='c')


cls_map = TaxonomyMap(taxonomy_name='PySimultan',
                      taxonomy_key='PySimultan',
                      taxonomy_entry_name='TestClass',
                      taxonomy_entry_key='TestClass',
                      content=[c_param_1, c_param_2, c_c],
                      )

mapper.register(cls_map.taxonomy_entry_key, TestClass, taxonomy_map=cls_map)
mapped_cls = mapper.get_mapped_class(cls_map.taxonomy_entry_key)


method_mapper.register_method(cls=mapped_cls,
                              name='test_method',
                              method=mapped_cls.test_method,
                              args=[],
                              kwargs={})


run_ui()



import typing
from SIMULTAN.Data import SimNamedObject_1, ISimManagedCollection, SimId, GeometricReference, SimManagedCollection_1, IReferenceLocation
from SIMULTAN.Data.SimMath import SimColor, SimPoint
from System import Guid, MulticastDelegate, IAsyncResult, AsyncCallback, ValueTuple_2, DateTime, IDisposable
from System.Reflection import MethodInfo
from System.Collections.ObjectModel import ObservableCollection_1
from System.ComponentModel import INotifyPropertyChanged
from SIMULTAN.Data.Components import SimComponentInstance, SimComponent
from System.Collections.Generic import Dictionary_2, List_1, IEnumerable_1
from SIMULTAN.Projects import ProjectData

class BaseSimNetworkElement(SimNamedObject_1[ISimManagedCollection], ISimNetworkElement):
    def __init__(self) -> None: ...
    @property
    def Color(self) -> SimColor: ...
    @Color.setter
    def Color(self, value: SimColor) -> SimColor: ...
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def Factory(self) -> ISimManagedCollection: ...
    @Factory.setter
    def Factory(self, value: ISimManagedCollection) -> ISimManagedCollection: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def Height(self) -> float: ...
    @Height.setter
    def Height(self, value: float) -> float: ...
    @property
    def Id(self) -> SimId: ...
    @Id.setter
    def Id(self, value: SimId) -> SimId: ...
    @property
    def IsConnected(self) -> bool: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def ParentNetwork(self) -> SimNetwork: ...
    @ParentNetwork.setter
    def ParentNetwork(self, value: SimNetwork) -> SimNetwork: ...
    @property
    def Ports(self) -> SimNetworkPort.SimNetworkPortCollection: ...
    @Ports.setter
    def Ports(self, value: SimNetworkPort.SimNetworkPortCollection) -> SimNetworkPort.SimNetworkPortCollection: ...
    @property
    def Position(self) -> SimPoint: ...
    @Position.setter
    def Position(self, value: SimPoint) -> SimPoint: ...
    @property
    def RepresentationReference(self) -> GeometricReference: ...
    @RepresentationReference.setter
    def RepresentationReference(self, value: GeometricReference) -> GeometricReference: ...
    @property
    def Width(self) -> float: ...
    @Width.setter
    def Width(self, value: float) -> float: ...
    def NotifyIsDeleted(self) -> None: ...
    def OnIsBeingDeleted(self) -> None: ...

    class IsBeingDeletedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any) -> None: ...


    class IsDeletedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any) -> None: ...


    class SimNetworkElementCollection(ObservableCollection_1[BaseSimNetworkElement]):
        def __init__(self, parentNetwork: SimNetwork) -> None: ...
        @property
        def Count(self) -> int: ...
        @property
        def Item(self) -> BaseSimNetworkElement: ...
        @Item.setter
        def Item(self, value: BaseSimNetworkElement) -> BaseSimNetworkElement: ...



class IElementWithComponent(INotifyPropertyChanged, typing.Protocol):
    @property
    def ComponentInstance(self) -> SimComponentInstance: ...
    @ComponentInstance.setter
    def ComponentInstance(self, value: SimComponentInstance) -> SimComponentInstance: ...
    @property
    def Id(self) -> SimId: ...
    @property
    def ParentNetwork(self) -> SimNetwork: ...


class INetwork(typing.Protocol):
    @property
    def HasParent(self) -> bool: ...


class ISimNetworkElement(typing.Protocol):
    pass


class PortType(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Input : PortType # 0
    Output : PortType # 1


class SimNetwork(BaseSimNetworkElement, INetwork):
    @typing.overload
    def __init__(self, name: str) -> None: ...
    @typing.overload
    def __init__(self, name: str, position: SimPoint) -> None: ...
    @property
    def Color(self) -> SimColor: ...
    @Color.setter
    def Color(self, value: SimColor) -> SimColor: ...
    @property
    def ContainedConnectors(self) -> SimNetworkConnector.SimNetworkConnectorCollection: ...
    @property
    def ContainedElements(self) -> BaseSimNetworkElement.SimNetworkElementCollection: ...
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def Factory(self) -> ISimManagedCollection: ...
    @Factory.setter
    def Factory(self, value: ISimManagedCollection) -> ISimManagedCollection: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def HasParent(self) -> bool: ...
    @property
    def Height(self) -> float: ...
    @Height.setter
    def Height(self, value: float) -> float: ...
    @property
    def Id(self) -> SimId: ...
    @Id.setter
    def Id(self, value: SimId) -> SimId: ...
    @property
    def IndexOfGeometricRepFile(self) -> int: ...
    @IndexOfGeometricRepFile.setter
    def IndexOfGeometricRepFile(self, value: int) -> int: ...
    @property
    def IsConnected(self) -> bool: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def ParentNetwork(self) -> SimNetwork: ...
    @ParentNetwork.setter
    def ParentNetwork(self, value: SimNetwork) -> SimNetwork: ...
    @property
    def Ports(self) -> SimNetworkPort.SimNetworkPortCollection: ...
    @Ports.setter
    def Ports(self, value: SimNetworkPort.SimNetworkPortCollection) -> SimNetworkPort.SimNetworkPortCollection: ...
    @property
    def Position(self) -> SimPoint: ...
    @Position.setter
    def Position(self, value: SimPoint) -> SimPoint: ...
    @property
    def RepresentationReference(self) -> GeometricReference: ...
    @RepresentationReference.setter
    def RepresentationReference(self, value: GeometricReference) -> GeometricReference: ...
    @property
    def Size(self) -> float: ...
    @Size.setter
    def Size(self, value: float) -> float: ...
    @property
    def Width(self) -> float: ...
    @Width.setter
    def Width(self, value: float) -> float: ...
    def Clone(self, name: str, factory: SimNetwork.SimNetworkCollection, parentNetwork: SimNetwork, portPairs: Dictionary_2[SimId, SimId]) -> ValueTuple_2[SimNetwork, Dictionary_2[SimId, SimId]]: ...
    def ConvertBlockToSubnetwork(self, block: SimNetworkBlock) -> SimNetwork: ...
    def GetAllBlocks(self) -> List_1[SimNetworkBlock]: ...
    def GetAllPorts(self) -> List_1[SimNetworkPort]: ...

    class AssociationChangedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, elements: IEnumerable_1[BaseSimNetworkElement], callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, elements: IEnumerable_1[BaseSimNetworkElement]) -> None: ...


    class SimNetworkCollection(SimManagedCollection_1[SimNetwork]):
        def __init__(self, owner: ProjectData) -> None: ...
        @property
        def CalledFromLocation(self) -> IReferenceLocation: ...
        @property
        def Count(self) -> int: ...
        @property
        def HasChanges(self) -> bool: ...
        @property
        def IsLoading(self) -> bool: ...
        @IsLoading.setter
        def IsLoading(self, value: bool) -> bool: ...
        @property
        def Item(self) -> SimNetwork: ...
        @Item.setter
        def Item(self, value: SimNetwork) -> SimNetwork: ...
        @property
        def LastChange(self) -> DateTime: ...
        @property
        def ProjectData(self) -> ProjectData: ...
        def DisconnectAllInstances(self, resource_id: int) -> None: ...
        def EndLoading(self) -> None: ...
        def GetElementsWithComponents(self) -> List_1[IElementWithComponent]: ...
        def RemoveItemSoft(self, item: SimNetwork) -> None: ...
        def StartLoading(self) -> None: ...



class SimNetworkBlock(BaseSimNetworkElement, IElementWithComponent):
    @typing.overload
    def __init__(self, name: str, position: SimPoint) -> None: ...
    @typing.overload
    def __init__(self, name: str, position: SimPoint, id: SimId, ports: IEnumerable_1[SimNetworkPort], color: SimColor) -> None: ...
    @property
    def Color(self) -> SimColor: ...
    @Color.setter
    def Color(self, value: SimColor) -> SimColor: ...
    @property
    def ComponentInstance(self) -> SimComponentInstance: ...
    @ComponentInstance.setter
    def ComponentInstance(self, value: SimComponentInstance) -> SimComponentInstance: ...
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def Factory(self) -> ISimManagedCollection: ...
    @Factory.setter
    def Factory(self, value: ISimManagedCollection) -> ISimManagedCollection: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def Height(self) -> float: ...
    @Height.setter
    def Height(self, value: float) -> float: ...
    @property
    def Id(self) -> SimId: ...
    @Id.setter
    def Id(self, value: SimId) -> SimId: ...
    @property
    def IsConnected(self) -> bool: ...
    @property
    def IsStatic(self) -> bool: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def ParentNetwork(self) -> SimNetwork: ...
    @ParentNetwork.setter
    def ParentNetwork(self, value: SimNetwork) -> SimNetwork: ...
    @property
    def Ports(self) -> SimNetworkPort.SimNetworkPortCollection: ...
    @Ports.setter
    def Ports(self, value: SimNetworkPort.SimNetworkPortCollection) -> SimNetworkPort.SimNetworkPortCollection: ...
    @property
    def Position(self) -> SimPoint: ...
    @Position.setter
    def Position(self, value: SimPoint) -> SimPoint: ...
    @property
    def RepresentationReference(self) -> GeometricReference: ...
    @RepresentationReference.setter
    def RepresentationReference(self, value: GeometricReference) -> GeometricReference: ...
    @property
    def Width(self) -> float: ...
    @Width.setter
    def Width(self, value: float) -> float: ...
    def AddRelativPositionToPortComp(self, portComponent: SimComponent, port: SimNetworkPort) -> None: ...
    def AssignComponent(self, component: SimComponent, syncBlockToComp: bool) -> None: ...
    def AttachEvents(self) -> None: ...
    def Clone(self, parent: SimNetwork) -> ValueTuple_2[SimNetworkBlock, Dictionary_2[SimId, SimId]]: ...
    def ConvertToStatic(self) -> None: ...
    def RemoveComponentInstance(self, sendAssociatedEvent: bool = ...) -> None: ...


class SimNetworkConnector(SimNamedObject_1[ISimManagedCollection], ISimNetworkElement):
    def __init__(self, port1: SimNetworkPort, port2: SimNetworkPort) -> None: ...
    @property
    def Color(self) -> SimColor: ...
    @Color.setter
    def Color(self, value: SimColor) -> SimColor: ...
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def Factory(self) -> ISimManagedCollection: ...
    @Factory.setter
    def Factory(self, value: ISimManagedCollection) -> ISimManagedCollection: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def Id(self) -> SimId: ...
    @Id.setter
    def Id(self, value: SimId) -> SimId: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def ParentNetwork(self) -> SimNetwork: ...
    @ParentNetwork.setter
    def ParentNetwork(self, value: SimNetwork) -> SimNetwork: ...
    @property
    def Points(self) -> ObservableCollection_1[SimPoint]: ...
    @Points.setter
    def Points(self, value: ObservableCollection_1[SimPoint]) -> ObservableCollection_1[SimPoint]: ...
    @property
    def RepresentationReference(self) -> GeometricReference: ...
    @RepresentationReference.setter
    def RepresentationReference(self, value: GeometricReference) -> GeometricReference: ...
    @property
    def Source(self) -> SimNetworkPort: ...
    @Source.setter
    def Source(self, value: SimNetworkPort) -> SimNetworkPort: ...
    @property
    def Target(self) -> SimNetworkPort: ...
    @Target.setter
    def Target(self, value: SimNetworkPort) -> SimNetworkPort: ...

    class SimNetworkConnectorCollection(ObservableCollection_1[SimNetworkConnector]):
        def __init__(self, parentNetwork: SimNetwork) -> None: ...
        @property
        def Count(self) -> int: ...
        @property
        def Item(self) -> SimNetworkConnector: ...
        @Item.setter
        def Item(self, value: SimNetworkConnector) -> SimNetworkConnector: ...



class SimNetworkPort(SimNamedObject_1[ISimManagedCollection], IElementWithComponent, IDisposable, ISimNetworkElement):
    @typing.overload
    def __init__(self, basePort: SimNetworkPort) -> None: ...
    @typing.overload
    def __init__(self, portType: PortType, name: str = ..., position: SimPoint = ...) -> None: ...
    @property
    def Color(self) -> SimColor: ...
    @Color.setter
    def Color(self, value: SimColor) -> SimColor: ...
    @property
    def ComponentInstance(self) -> SimComponentInstance: ...
    @ComponentInstance.setter
    def ComponentInstance(self, value: SimComponentInstance) -> SimComponentInstance: ...
    @property
    def ConnectedPorts(self) -> List_1[SimNetworkPort]: ...
    @property
    def Connectors(self) -> List_1[SimNetworkConnector]: ...
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def Factory(self) -> ISimManagedCollection: ...
    @Factory.setter
    def Factory(self, value: ISimManagedCollection) -> ISimManagedCollection: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def Id(self) -> SimId: ...
    @Id.setter
    def Id(self, value: SimId) -> SimId: ...
    @property
    def IsConnected(self) -> bool: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def ParentNetwork(self) -> SimNetwork: ...
    @property
    def ParentNetworkElement(self) -> BaseSimNetworkElement: ...
    @ParentNetworkElement.setter
    def ParentNetworkElement(self, value: BaseSimNetworkElement) -> BaseSimNetworkElement: ...
    @property
    def PortType(self) -> PortType: ...
    @PortType.setter
    def PortType(self, value: PortType) -> PortType: ...
    @property
    def Position(self) -> SimPoint: ...
    @Position.setter
    def Position(self, value: SimPoint) -> SimPoint: ...
    @property
    def RepresentationReference(self) -> GeometricReference: ...
    @RepresentationReference.setter
    def RepresentationReference(self, value: GeometricReference) -> GeometricReference: ...
    def ConnectTo(self, port: SimNetworkPort) -> None: ...
    def Dispose(self) -> None: ...
    def RemoveConnections(self) -> None: ...

    class SimNetworkPortCollection(ObservableCollection_1[SimNetworkPort]):
        def __init__(self, parentElement: BaseSimNetworkElement) -> None: ...
        @property
        def Count(self) -> int: ...
        @property
        def Item(self) -> SimNetworkPort: ...
        @Item.setter
        def Item(self, value: SimNetworkPort) -> SimNetworkPort: ...





import typing, abc
from System.IO import FileInfo
from System.ComponentModel import INotifyPropertyChanged
from System import Guid, Array_1, ValueTuple_2
from System.Security import SecureString
from System.Collections.ObjectModel import ObservableCollection_1

class IAuthenticationService(typing.Protocol):
    @abc.abstractmethod
    def Authenticate(self, userManager: SimUsersManager, projectFile: FileInfo) -> SimUser: ...


class SimUser(INotifyPropertyChanged):
    def __init__(self, id: Guid, name: str, passwordHash: Array_1[int], encryptedEncryptionKey: Array_1[int], role: SimUserRole) -> None: ...
    @classmethod
    @property
    def DefaultUser(cls) -> SimUser: ...
    @classmethod
    @property
    def DefaultUserPassword(cls) -> SecureString: ...
    @property
    def EncryptedEncryptionKey(self) -> Array_1[int]: ...
    @EncryptedEncryptionKey.setter
    def EncryptedEncryptionKey(self, value: Array_1[int]) -> Array_1[int]: ...
    @property
    def Id(self) -> Guid: ...
    @Id.setter
    def Id(self, value: Guid) -> Guid: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def PasswordHash(self) -> Array_1[int]: ...
    @PasswordHash.setter
    def PasswordHash(self, value: Array_1[int]) -> Array_1[int]: ...
    @property
    def Role(self) -> SimUserRole: ...
    @Role.setter
    def Role(self, value: SimUserRole) -> SimUserRole: ...


class SimUserRole(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    ADMINISTRATOR : SimUserRole # 0
    MODERATOR : SimUserRole # 1
    ENERGY_NETWORK_OPERATOR : SimUserRole # 2
    ENERGY_SUPPLIER : SimUserRole # 3
    BUILDING_DEVELOPER : SimUserRole # 4
    BUILDING_OPERATOR : SimUserRole # 5
    ARCHITECTURE : SimUserRole # 6
    FIRE_SAFETY : SimUserRole # 7
    BUILDING_PHYSICS : SimUserRole # 8
    MEP_HVAC : SimUserRole # 9
    PROCESS_MEASURING_CONTROL : SimUserRole # 10
    BUILDING_CONTRACTOR : SimUserRole # 11
    GUEST : SimUserRole # 12


class SimUsersManager(INotifyPropertyChanged):
    def __init__(self) -> None: ...
    @property
    def CurrentUser(self) -> SimUser: ...
    @CurrentUser.setter
    def CurrentUser(self, value: SimUser) -> SimUser: ...
    @property
    def EncryptionKey(self) -> Array_1[int]: ...
    @EncryptionKey.setter
    def EncryptionKey(self, value: Array_1[int]) -> Array_1[int]: ...
    @property
    def Users(self) -> ObservableCollection_1[SimUser]: ...
    @Users.setter
    def Users(self, value: ObservableCollection_1[SimUser]) -> ObservableCollection_1[SimUser]: ...
    def Clear(self) -> None: ...
    @staticmethod
    def DecryptEncryptionKey(encryptedKey: Array_1[int], password: Array_1[int]) -> Array_1[int]: ...
    def DecryptUserSecret(self, encryptedSecret: str) -> ValueTuple_2[str, bool]: ...
    def EncryptUserSecret(self, secret: str) -> str: ...
    # Skipped Authenticate due to it being static, abstract and generic.

    Authenticate : Authenticate_MethodGroup
    class Authenticate_MethodGroup:
        @typing.overload
        def __call__(self, name: str, password: SecureString) -> ValueTuple_2[SimUser, Array_1[int]]:...
        @typing.overload
        def __call__(self, user: SimUser, password: SecureString) -> bool:...
        @typing.overload
        def __call__(self, name: str, password: Array_1[int], isHashedPassword: bool = ...) -> SimUser:...
        @typing.overload
        def __call__(self, user: SimUser, password: Array_1[int], isHashedPassword: bool = ...) -> bool:...

    # Skipped EncryptEncryptionKey due to it being static, abstract and generic.

    EncryptEncryptionKey : EncryptEncryptionKey_MethodGroup
    class EncryptEncryptionKey_MethodGroup:
        @typing.overload
        def __call__(self, unencryptedKey: Array_1[int], password: Array_1[int]) -> Array_1[int]:...
        @typing.overload
        def __call__(self, unencryptedKey: Array_1[int], password: SecureString) -> Array_1[int]:...

    # Skipped HashPassword due to it being static, abstract and generic.

    HashPassword : HashPassword_MethodGroup
    class HashPassword_MethodGroup:
        @typing.overload
        def __call__(self, password: Array_1[int]) -> Array_1[int]:...
        @typing.overload
        def __call__(self, password: SecureString) -> Array_1[int]:...





import numpy as np
import pandas as pd

from PySimultan2.data_model import DataModel
from PySimultan2 import FileInfo

from PySimultan2.object_mapper import PythonMapper
from PySimultan2.taxonomy_maps import TaxonomyMap, Content

import resources

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 importlib_resources.
    import importlib_resources as pkg_resources


with pkg_resources.path(resources, 'test_set_value_project.simultan') as r_path:
    project_path = str(r_path)

data_model = DataModel.create_new_project(project_path=project_path,
                                          user_name='admin',
                                          password='admin')

mapper = PythonMapper()


def map_classes():
    class TestComponent(object):
        def __init__(self, *args, **kwargs):
            self.value = kwargs.get('mapped_property')

    content0 = Content(text_or_key='value',  # text or key of the content/parameter/property
                       property_name='mapped_property',  # name of the generated property
                       type=None,  # type of the content/parameter/property
                       unit=None,  # unit of the content/parameter/property
                       documentation='mapped_property to test',
                       component_policy='subcomponent')

    content1 = Content(text_or_key='value2',  # text or key of the content/parameter/property
                       property_name='value2',  # name of the generated property
                       type=None,  # type of the content/parameter/property
                       unit=None,  # unit of the content/parameter/property
                       documentation='value2',
                       component_policy='reference')

    test_component_map = TaxonomyMap(taxonomy_name='PySimultan',
                                     taxonomy_key='PySimultan',
                                     taxonomy_entry_name='TestComponent',
                                     taxonomy_entry_key='TestComponent',
                                     content=[content0, content1],
                                     )

    mapper.register(test_component_map.taxonomy_entry_key, TestComponent, taxonomy_map=test_component_map)
    cls1 = mapper.get_mapped_class(test_component_map.taxonomy_entry_key)

    test_component_map_2 = TaxonomyMap(taxonomy_name='PySimultan',
                                       taxonomy_key='PySimultan',
                                       taxonomy_entry_name='TestComponent2',
                                       taxonomy_entry_key='TestComponent2',
                                       content=[content0],
                                       )

    mapper.register(test_component_map_2.taxonomy_entry_key, TestComponent, taxonomy_map=test_component_map_2)
    cls2 = mapper.get_mapped_class(test_component_map_2.taxonomy_entry_key)

    return cls1, cls2


mapped_cls_1, mapped_cls_2 = map_classes()
component = mapped_cls_1(name='test_component',
                         value=None)


def test_set_parameter(test_component):
    test_component.mapped_property = 'test_value'
    assert test_component.mapped_property == 'test_value'

    test_component.mapped_property = 1
    assert test_component.mapped_property == 1

    test_component.mapped_property = 1.0
    assert test_component.mapped_property == 1.0

    test_component.mapped_property = True
    assert test_component.mapped_property


def test_set_set_file(test_component):

    with pkg_resources.path(resources, 'new_test_file_1.txt') as r_path:
        with open(str(r_path), 'w') as f:
            f.write('This is the content of the file 1.')

        file_info_1 = FileInfo(file_path=r_path)

    with pkg_resources.path(resources, 'new_test_file_2.txt') as r_path:
        with open(str(r_path), 'w') as f:
            f.write('This is the content of the file 2.')

        file_info_2 = FileInfo(file_path=r_path)

    test_component.mapped_property = file_info_1
    assert test_component.mapped_property.content == 'This is the content of the file 1.'

    test_component.mapped_property = file_info_2
    assert test_component.mapped_property.content == 'This is the content of the file 2.'


def test_set_value_field(test_component):

    df = pd.DataFrame(np.random.rand(6, 4),
                      columns=list("ABCD"))

    numpy_value = np.arange(1000, dtype=float).reshape(10, 10, 10)

    test_component.mapped_property = numpy_value
    assert np.array_equal(test_component.mapped_property, numpy_value)

    test_component.mapped_property = df
    assert np.array_equal(test_component.mapped_property.values, df.values)


def set_referenced_component(test_component):
    referenced_component = mapped_cls_2(name='referenced_component',
                                        value=None)
    test_component.mapped_property = referenced_component
    assert test_component.mapped_property._wrapped_obj.Id.Equals(referenced_component.Id)


def test_set_dictionary_param(test_component):
    test_dict = {'val_a': 1,
                 'val_b': 2.0,
                 'val_c': 'test'}
    test_component.mapped_property = test_dict
    test_component.mapped_property['val_a'] = 222

    assert test_component.mapped_property['val_a'] == 222


def set_new_component(test_component):
    new_component = data_model.create_new_component('SubComponent',
                                                    add_to_project=False)
    test_component.mapped_property = new_component
    assert test_component.mapped_property._wrapped_obj.Id.Equals(new_component.Id)


if __name__ == '__main__':
    test_set_parameter(component)
    test_set_dictionary_param(component)
    test_set_set_file(component)
    test_set_value_field(component)
    set_referenced_component(component)
    set_new_component(component)
    data_model.save()

data_model.cleanup()
print('Test passed')
